<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;


/**
 * Template style controller class.
 *
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       1.6
 */
class dropeditorControllerButton extends JControllerLegacy
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 * @since   3.2
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		// Apply, Save & New, and Save As copy should be standard on forms.
		$this->registerTask('apply', 'save');
	}

	/**
	 * Method for closing the template.
	 *
	 * @return  void.
	 *
	 * @since   3.2
	 */
	public function cancel()
	{
		$this->setRedirect(JRoute::_('index.php?option=com_dropeditor', false));
	}

	

	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional (note, the empty array is atypical compared to other models).
	 *
	 * @return  JModelLegacy  The model.
	 *
	 * @since   3.2
	 */
	public function getModel($name = 'Button', $prefix = 'DropeditorModel', $config = array())
	{
		$model = parent::getModel($name, $prefix, $config);

		return $model;
	}

	/**
	 * Method to check if you can add a new record.
	 *
	 * @return  boolean
	 *
	 * @since   3.2
	 */
	protected function allowEdit()
	{
		return JFactory::getUser()->authorise('core.edit', 'com_dropeditor');
	}

	/**
	 * Method to check if you can save a new or existing record.
	 *
	 * @return  boolean
	 *
	 * @since   3.2
	 */
	protected function allowSave()
	{
		return $this->allowEdit();
	}

    public function add()
	{
        $app          = JFactory::getApplication();
        $data         = $this->input->post->get('jform', array(), 'array');

        $model = $this->getModel();
        $newItem = $model->addButton($data);
        if($newItem){
            $item = $model->getButton($newItem);
            $this->exit_status(true,array('id'=>$newItem,'title'=> $item->title));
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
	public function delete(){
        $id = JFactory::getApplication()->input->getInt('id_button');
        $model = $this->getModel();
        if($model->deleteButton($id)){
            $this->exit_status(true);
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
    public function update($key = null, $urlVar = null)
	{
        $id = JRequest::getInt('id',0);
        $data = JRequest::getVar('jform', array());
        $model = $this->getModel();
        $data['id'] = $id;
        if($model->saveButton($data)){
            //also save to editor config
            $this->exit_status(true);
        }else{
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
	}
        
	/**
     * Return a json response
     * @param $status
     * @param array $datas array of datas to return with the json string
     *
     */
	private function exit_status($status,$datas=array()){
        $response = array('status'=>$status,'datas'=>$datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
	}
}
