<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

jimport('joomla.application.component.controllerform');
/**
 * Template style controller class.
 *
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       1.6
 */
class dropeditorControllerFormat extends JControllerForm
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 * @since   3.2
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		// Apply, Save & New, and Save As copy should be standard on forms.
		$this->registerTask('apply', 'save');
	}

	

	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional (note, the empty array is atypical compared to other models).
	 *
	 * @return  JModelLegacy  The model.
	 *
	 * @since   3.2
	 */
	public function getModel($name = 'Format', $prefix = 'DropeditorModel', $config = array())
	{
		$model = parent::getModel($name, $prefix, $config);

		return $model;
	}

    public function load() {
        $id = JFactory::getApplication()->input->getInt("id",0);
        if($id) {

            $model = $this->getModel();
            $style = $model->getItem($id);
            if($style) {
                $this->exit_status(true, $style);
            }else {
                $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
            }
        } else {
            $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
      
    public function deleteStyle(){
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        if($model->deleteStyle($id)){
            $this->exit_status(true);
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
    public function copyStyle() {
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        $newItem = $model->copyStyle($id);
        if($newItem){
            $item = $model->getCustomStyle($newItem);
            $this->exit_status(true,array('id'=> $item->id,'name'=> $item->name));
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
   
	public function setStyleClass(){
        $id = JRequest::getInt('id',0);
        
        $model = $this->getModel();
        $canDo = DropeditorHelper::getActions();
        if(!$canDo->get('core.edit')){
            if($canDo->get('core.edit.own')){
                
            }else{
                $this->exit_status('not permitted');
            }
        }
        
        $name = JFactory::getApplication()->input->getString('name');
        //make a safe name
        $name = preg_replace('/([^a-zA-Z0-9-]+)/', '_', $name);
        if($model->setStyleClass($id,$name)){
             DropeditorHelper::writeCustomStyle($model->getCustomStyles());
            $this->exit_status(true, array('name'=> $name));
        }else{
             $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
    
    public function addCustomStyle() {
        $model = $this->getModel();
        $newItem = $model->addCustomStyle();
        if($newItem){
            $item = $model->getCustomStyle($newItem);
            $this->exit_status(true,array('id'=>$newItem,'name'=> $item->name));
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
	public function loadCustomStyle() {
	    $id = JFactory::getApplication()->input->getInt("id",0);
	    if($id) {

	        $model = $this->getModel();
	        $style = $model->getCustomStyle($id);
	        if($style) {
	            $this->exit_status(true, $style);
	        }else {
	            $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
	        }
	    } else {
	        $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
	    }
	}
        
	/**
	 * Saves a template source file.
	 *
	 * @return  void
	 *
	 * @since   3.2
	 */
	public function saveCustomStyle($key = null, $urlVar = null)
	{
        $id = JRequest::getInt('id',0);
        $data = JRequest::getVar('jform', array());
        $model = $this->getModel();
        $data['id'] = $id;
        if($model->saveCustomStyle($data)){
            DropeditorHelper::writeCustomStyle($model->getCustomStyles());
            $this->exit_status(true);
        }else{
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
        		
	}
        
	/**
    * Return a json response
    * @param $status
    * @param array $datas array of datas to return with the json string
    *
    */
    private function exit_status($status,$datas=array()){
    	$response = array('status'=>$status,'datas'=>$datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
   


	/**
	 * Saves a template source file.
	 *
	 * @return  void
	 *
	 * @since   3.2
	 */
	public function save($key = null, $urlVar = null)
	{
		$id = JRequest::getInt('id',0);
        $data = JRequest::getVar('jform', array());
        $model = $this->getModel();
        $data['id'] = $id;
        if($model->save($data)){
            //also save to editor config
            DropeditorHelper::writeTitlesOption($model->getItems());
            $this->exit_status(true);
        }else{
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
	}
}
