<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

JLoader::register('InstallerModelInstall', JPATH_ADMINISTRATOR . '/components/com_installer/models/install.php');

/**
 * Template style controller class.
 *
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       1.6
 */
class dropeditorControllerHtmltemplates extends JControllerLegacy
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 * @since   3.2
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		// Apply, Save & New, and Save As copy should be standard on forms.
		$this->registerTask('apply', 'save');
	}

	/**
	 * Method for closing the template.
	 *
	 * @return  void.
	 *
	 * @since   3.2
	 */
	public function cancel()
	{
		$this->setRedirect(JRoute::_('index.php?option=com_dropeditor', false));
	}

	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional (note, the empty array is atypical compared to other models).
	 *
	 * @return  JModelLegacy  The model.
	 *
	 * @since   3.2
	 */
	public function getModel($name = 'Htmltemplates', $prefix = 'DropeditorModel', $config = array())
	{
		$model = parent::getModel($name, $prefix, $config);

		return $model;
	}

	/**
	 * Method to check if you can add a new record.
	 *
	 * @return  boolean
	 *
	 * @since   3.2
	 */
	protected function allowEdit()
	{
		return JFactory::getUser()->authorise('core.edit', 'com_dropeditor');
	}

	/**
	 * Method to check if you can save a new or existing record.
	 *
	 * @return  boolean
	 *
	 * @since   3.2
	 */
	protected function allowSave()
	{
		return $this->allowEdit();
	}
        
	public function load() {
	    $id = JFactory::getApplication()->input->getInt("id",0);
	    if($id) {
	        $model = $this->getModel();
	        $item = $model->getTemplate($id);
	        if($item) {
	            $this->exit_status(true, $item);
	        }else {
	            $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
	        }
	    } else {
	        $this->exit_status( JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
	    }
	}
           
    public function delete(){
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        if($model->deleteTemplate($id)){
            $this->exit_status(true);
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }
    }
        
    public function add() {
        $model = $this->getModel();
        $newItem = $model->addTemplate();
        if($newItem){
            $item = $model->getTemplate($newItem);
            $this->exit_status(true,array('id'=>$newItem,'name'=> $item->title));
        }else {
            $this->exit_status(JText::_('COM_DROPEDITOR_GLOBAL_ERROR'));
        }

    }
        
    private function exit_status($status,$datas=array()){
        $response = array('status'=>$status,'datas'=>$datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
