<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * Import/Export controller class
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       2.2
 */
class DropeditorControllerImportExport extends JControllerLegacy
{
	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional (note, the empty array is atypical compared to other models).
	 *
	 * @return  JModelLegacy  The model.
	 *
	 * @since   3.2
	 */
	public function getModel($name = 'ImportExport', $prefix = 'DropeditorModel', $config = array())
	{
		$model = parent::getModel($name, $prefix, $config);

		return $model;
	}

	/**
	 * Method to export profile or custom styles to a json file
	 */
	public function export() {
		$app = JFactory::getApplication();
		$inputs = $app->input;
		$form_actions = $inputs->get('form-actions', null, 'RAW');
		$profile_id_list = $inputs->get('profile-id', '', 'RAW');
		if ($profile_id_list) {
			$profile_id_list = implode(',', $profile_id_list);
		}
		$model = $this->getModel();
		$data = array();

		if ($form_actions == 'export-profiles') {
			$data = $model->getData('profiles', $profile_id_list);
			if (empty($data)) {
				$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_EXPORT_ERROR_NO_PROFILES_SELECTED'), 'error');
				exit;
			}
			header( 'Content-Disposition: attachment; filename=dropeditor_profiles_export-' . date( 'm-d-Y' ) . '.json' );
		} elseif ($form_actions == 'export-styles') {
			$data = $model->getData('custom_styles');
			header( 'Content-Disposition: attachment; filename=dropeditor_custom_styles_export-' . date( 'm-d-Y' ) . '.json' );
		}

		// Throw error to user
		if (empty($data)) {
			$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_EXPORT_ERROR'), 'error');
			exit;
		}

		header( 'Content-Type: application/json; charset=utf-8' );
		header( "Expires: 0" );
		echo base64_encode(json_encode($data));
		exit;
	}

	/**
	 * Method to import profile or custom styles from a json file
	 */
	public function import() {
		$model = $this->getModel();
		$app = JFactory::getApplication();
		$file = $app->input->files->get('import_file');
		$ext = JFile::getExt($file['name']);
		if ($ext != 'json') {
			$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_ERROR_WRONG_FILE'), 'error');
			exit;
		}

		$file_contents = file_get_contents($file['tmp_name']);
		if ($file_contents == '') {
			$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_ERROR_EMPTY_FILE'), 'error');
			exit;
		}

		$file_contents = base64_decode($file_contents);
		$file_contents = json_decode($file_contents, true);

		if (!isset($file_contents['file_type']) || ($file_contents['file_type'] != 'profiles' && $file_contents['file_type'] != 'custom_styles')) {
			$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_ERROR_MODIFIED_FILE'), 'error');
			exit;
		}

		switch ($file_contents['file_type']) {
			case 'profiles':
				unset($file_contents['file_type']);

				if (!$model->importProfiles($file_contents)) {
					$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_ERROR'), 'error');
				} else {
					$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_SUCCESS_PROFILES'));
				}

				exit;
				break;
			case 'custom_styles':
				unset($file_contents['file_type']);
				$override = $app->input->getInt('override-styles', 0);

				if (!$model->importCustomStyles($file_contents, $override)) {
					$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_ERROR'), 'error');
				} else {
					$app->redirect('index.php?option=com_dropeditor&view=importexport&tmpl=component', JText::_('COM_DROPEDITOR_VIEW_IMPORT_EXPORT_IMPORT_SUCCESS_STYLES'));
				}

				exit;
				break;
		}
	}
}