<?php
/** 
 * Dropfiles
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.controllerform');
jimport( 'joomla.filesystem.folder' );
jimport( 'joomla.filesystem.file' );

class DropfilesControllerConfig extends JControllerForm {
    
	public function save($key = null, $urlVar = null)
	{
		// Check for request forgeries.
		JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));

		// Initialise variables.
		$app   = JFactory::getApplication();
		$lang  = JFactory::getLanguage();
		$model = $this->getModel();
		$data  = JRequest::getVar('jform', array(), 'post', 'array');
		$context = "$this->option.edit.$this->context";
	
		// Access check.
                $canDo = DropfilesHelper::getActions();
                if(!$canDo->get('core.edit')){
                    if($canDo->get('core.edit.own')){
                        $category = $model->getItem(JFactory::getApplication()->input->getInt('id', 0));
                        if($category->created_user_id != JFactory::getUser()->id){
                            $this->exit_status('not permitted');
                        }
                    }else{
                        $this->exit_status('not permitted');
                    }
                }
                
		// Validate the posted data.
		// Sometimes the form needs some posted data, such as for plugins and modules.
		$form = $model->getForm($data, false);

		if (!$form)
		{
			$app->enqueueMessage($model->getError(), 'error');

			return false;
		}

		// Test whether the data is valid.
		$validData = $model->validate($form, $data);

		// Check for validation errors.
		if ($validData === false)
		{
			// Get the validation messages.
			$errors = $model->getErrors();

			// Push up to three validation messages out to the user.
			for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
			{
				if ($errors[$i] instanceof Exception)
				{
					$app->enqueueMessage($errors[$i]->getMessage(), 'warning');
				}
				else
				{
					$app->enqueueMessage($errors[$i], 'warning');
				}
			}

			// Save the data in the session.
			$app->setUserState($context . '.data', $data);

			// Redirect back to the edit screen.
			$this->setRedirect(
				JRoute::_(
					'index.php?option=' . $this->option . '&view=' . $this->view_item
					. $this->getRedirectToItemAppend($recordId, $urlVar), false
				)
			);

			return false;
		}

		// Attempt to save the data.
		if (!$model->save($validData))
		{
			// Save the data in the session.
			$app->setUserState($context . '.data', $validData);

			// Redirect back to the edit screen.
			$this->setError(JText::sprintf('JLIB_APPLICATION_ERROR_SAVE_FAILED', $model->getError()));
			$this->setMessage($this->getError(), 'error');

			$this->setRedirect(
				JRoute::_(
					'index.php?option=' . $this->option . '&view=' . $this->view_item.'&tmpl=component'
					. $this->getRedirectToItemAppend($recordId, $urlVar), false
				)
			);

			return false;
		}

	

		$this->setMessage(
			JText::_(
				($lang->hasKey($this->text_prefix . ($recordId == 0 && $app->isSite() ? '_SUBMIT' : '') . '_SAVE_SUCCESS')
					? $this->text_prefix
					: 'JLIB_APPLICATION') . ($recordId == 0 && $app->isSite() ? '_SUBMIT' : '') . '_SAVE_SUCCESS'
			)
		);

                $app->setUserState($context . '.data', null);

                // Redirect to the list screen.
                $this->setRedirect(
                        JRoute::_(
                                'index.php?option=' . $this->option . '&view=' . $this->view_item.'&tmpl=component'
                                . $this->getRedirectToListAppend(), false
                        )
                );

		// Invoke the postSave method to allow for the child class to access the model.
		$this->postSaveHook($model, $validData);

		return true;
	}

        
        public function getRedirectToItemAppend($recordId = NULL, $urlVar = 'id'){
            $append = parent::getRedirectToItemAppend($recordId, $urlVar);

            $format = JRequest::getCmd('format', 'raw');

            // Setup redirect info.
            if ($format)
            {
                    $append .= '&format=' . $format;
            }
            return $append; 
        }    

        protected function allowEdit($data = array(), $key = 'id')
        {
            return true;
        }
        
        public function setTheme(){
            $theme = JRequest::getCmd('theme');
            $id = JRequest::getInt('id');
            
            $canDo = DropfilesHelper::getActions();
            if(!$canDo->get('core.edit')){
                if($canDo->get('core.edit.own')){
                    $modelC = $this->getModel('category');
                    $category = $modelC->getItem($id);
                    if($category->created_user_id != JFactory::getUser()->id){
                        $this->exit_status('not permitted');
                    }
                }else{
                    $this->exit_status('not permitted');
                }
            }
            
            JPluginHelper::importPlugin('dropfilesthemes');
            $dispatcher = JDispatcher::getInstance();
            $themesObj = $dispatcher->trigger('getThemeName');

            $themes = array();
            foreach ($themesObj as $value) {
                $themes[] = $value['id'];
            }
            
            if(!in_array($theme, $themes)){
                $theme = 'default';
            }
            
            $model = $this->getModel();
            if($model->setTheme($theme,$id)){
                $result = true;
            }else{
                $result = false;
            }
            echo json_encode($result);
            JFactory::getApplication()->close();
        }
        
	/**
        * Return a json response
        * @param $status
        * @param array $datas array of datas to return with the json string
        * 
        */
       private function exit_status($status,$datas=array()){
               $response = array('response'=>$status,'datas'=>$datas);            
               echo json_encode($response);
               JFactory::getApplication()->close();
       }

    public function getDropboxToken() {

        $code = JFactory::getApplication()->input->get('authencode');

        $canDo = DropfilesHelper::getActions();

        if(!$canDo->get('core.edit')){
            $this->exit_status('not permitted');
        }

        $Dropbox = new dropfilesDropbox();


        $list = $Dropbox->convertAuthorizationCode($code);


        if($list['accessToken']){
            $Dropbox->saveCodeToken($code, $list['accessToken']);

        }
        echo json_encode(array());die;
    }

    public function logoutDropbox() {

        $canDo = DropfilesHelper::getActions();

        if(!$canDo->get('core.edit')){
            $this->setRedirect(
                JRoute::_(
                    'index.php?option=com_config&view=component&component=com_dropfiles', false
                ), JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED')
            );
            $this->redirect();
        }


        $Dropbox = new dropfilesDropbox();
        $Dropbox->logout();

        $this->setRedirect(
            JRoute::_(
                'index.php?option=com_config&view=component&component=com_dropfiles', false
            )
        );
        $this->redirect();
    }

    /**
     *
     * Import jdownloads files from selected category
     * @since version
     */
    public function downimport() {


        $path = JPATH_SITE.'/components/com_jdownloads/helpers/categories.php';
        if (is_file($path)) include_once $path;

        include_once JPATH_ADMINISTRATOR . '/components/com_dropfiles/controllers/category.php';

        $params = JComponentHelper::getParams('com_dropfiles');

        $allowed_ext = explode(',', $params->get('allowedext','7z,ace,bz2,dmg,gz,rar,tgz,zip,csv,doc,docx,html,key,keynote,odp,ods,odt,pages,pdf,pps,ppt,pptx,rtf,tex,txt,xls,xlsx,xml,bmp,exif,gif,ico,jpeg,jpg,png,psd,tif,tiff,aac,aif,aiff,alac,amr,au,cdda,flac,m3u,m4a,m4p,mid,mp3,mp4,mpa,ogg,pac,ra,wav,wma,3gp,asf,avi,flv,m4v,mkv,mov,mpeg,mpg,rm,swf,vob,wmv'));
        foreach ($allowed_ext as $key => $value) {
            $allowed_ext[$key] = strtolower(trim($allowed_ext[$key]));
            if($allowed_ext[$key]==''){
                unset($allowed_ext[$key]);
            }
        }

        $id = JFactory::getApplication()->input->getInt('doccat');

        JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_jdownloads'.DS.'tables');
        JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_jdownloads/models/','jdownloadsModel');


        $categories = JDCategories::getInstance('jdownloads', '');
        $cat = $categories->get($id);

        if (is_object($cat)) {

            $db        = JFactory::getDbo();
            $query     = $db->getQuery(true);

            $query->select('c.*');
            $query->from('`#__jdownloads_categories` AS c');

            $rgt = $cat->rgt;
            $lft = $cat->lft;
            $query->where('c.lft >= '.(int) $lft);
            $query->where('c.rgt <= '.(int) $rgt);
            $db->setQuery($query);
            $rows = $db->loadObjectList();

            $parent_id = 1;
            if (count($rows)) {
                $catcontroller = new DropfilesControllerCategory();

                $catpath = '';
                foreach ($rows as $category) {

                    $db        = JFactory::getDbo();
                    $query     = $db->getQuery(true);

                    // Select the required fields from the table.
                    $query->select(
                            'a.file_id, a.file_title, a.file_alias, a.description, a.file_pic, a.price, a.release, a.cat_id, '.
                            'a.size, a.date_added, a.publish_from, a.publish_to, a.use_timeframe, a.url_download, a.other_file_id, a.extern_file, a.downloads, '.
                            'a.extern_site, a.notes, a.access, a.language, a.checked_out, a.checked_out_time, a.ordering, a.featured, a.published, a.asset_id'
                    );
                    $query->from('`#__jdownloads_files` AS a');

                    // Join over the users for the checked out user.
                    $query->select('uc.name AS editor');
                    $query->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

                    // Join over the files for other selected file
                    $query->select('f.url_download AS other_file_name, f.file_title AS other_download_title');
                    $query->join('LEFT', $db->quoteName('#__jdownloads_files').' AS f ON f.file_id = a.other_file_id');

                    // Join over the language
                    $query->select('l.title AS language_title');
                    $query->join('LEFT', $db->quoteName('#__languages').' AS l ON l.lang_code = a.language');

                    // Join over the asset groups.
                    $query->select('ag.title AS access_level');
                    $query->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');

                    // Join over the categories.
                    $query->select('c.title AS category_title, c.parent_id AS category_parent_id');
                    $query->join('LEFT', '#__jdownloads_categories AS c ON c.id = a.cat_id');

                    $query->where('a.published = 1');


                    $query->where('a.cat_id = '.$category->id);
                    $db->setQuery($query);

                    $downloads = $db->loadObjectList();


                    $downloaddir = JPATH_ROOT . '/jdownloads';

                    $datas = array();
                    $datas['jform']['extension'] = 'com_dropfiles';
                    $datas['jform']['title'] = $category->title;
                    $datas['jform']['alias'] = $category->alias. '-' . date('dmY-h-m-s',time());
                    $datas['jform']['parent_id'] = $parent_id;
                    $datas['jform']['published'] = 1;
                    $datas['jform']['language'] = '*';
                    $datas['jform']['metadata']['tags'] = '';

                    //Set state value to retreive the correct table
                    $model = $this->getModel('category');

                    $model->setState('category.extension', 'category');

                    foreach ($datas as $data => $val) {
                        JRequest::setVar($data, $val, 'POST');
                    }

                    if($catcontroller->save())
                    {
                        $newId = $catcontroller->savedId;
                        $parent_id = $newId;

                        $file_dir = dropfilesBase::getFilesPath($newId);
                        if(!file_exists($file_dir)){
                            JFolder::create($file_dir);
                            $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                            JFile::write($file_dir.'index.html', $data);
                            $data = 'deny from all';
                            JFile::write($file_dir.'.htaccess', $data);
                        }

                        $modelFiles = $this->getModel('files');
                        $user = JFactory::getUser();

                        $catpath .= '/' . $category->title;
                        foreach ($downloads as $download) {

                            if ($download->url_download != '') {

                                if(!in_array(strtolower(JFile::getExt($download->url_download)),$allowed_ext)){
                                    continue;
                                }

                                $newname = uniqid().'.'.strtolower(JFile::getExt($download->url_download));

                                $downloadfile = $downloaddir . $catpath  . '/' . $download->url_download;

                                if (file_exists($downloadfile)) {
                                    JFile::copy($downloadfile, $file_dir . $newname);
                                }

                                //Insert new image into databse

                                $id_file = $modelFiles->addFile(array(
                                    'title' => $download->file_title,
                                    'description' => $download->description,
                                    'id_category' => $newId,
                                    'file' => $newname,
                                    'ext' => strtolower(JFile::getExt($download->url_download)),
                                    'size' => filesize($file_dir.$newname),
                                    'author' => $user->get('id')
                                ));
                                if(!$id_file){
                                    JFile::delete($file_dir.$newname);
                                }

                            } else {

                                $urlfile  = pathinfo($download->extern_file);
                                $id_file = $modelFiles->addFile(array(
                                    'title' => $download->file_title,
                                    'description' => $download->description,
                                    'id_category' => $newId,
                                    'file' => $download->extern_file,
                                    'ext' => $urlfile['extension'],
                                    'size' => $this->remote_file_size($download->extern_file),
                                    'author' => $user->get('id')
                                ));
                            }

                        }

                    }
                }
            }

        }


        $this->exit_status('Done');

    }

    /**
     *  Import Files fromt eDocman
     *
     * @since version
     */
    public function edocimport() {

        include_once JPATH_ADMINISTRATOR . '/components/com_dropfiles/controllers/category.php';

        $params = JComponentHelper::getParams('com_dropfiles');

        $allowed_ext = explode(',', $params->get('allowedext','7z,ace,bz2,dmg,gz,rar,tgz,zip,csv,doc,docx,html,key,keynote,odp,ods,odt,pages,pdf,pps,ppt,pptx,rtf,tex,txt,xls,xlsx,xml,bmp,exif,gif,ico,jpeg,jpg,png,psd,tif,tiff,aac,aif,aiff,alac,amr,au,cdda,flac,m3u,m4a,m4p,mid,mp3,mp4,mpa,ogg,pac,ra,wav,wma,3gp,asf,avi,flv,m4v,mkv,mov,mpeg,mpg,rm,swf,vob,wmv'));
        foreach ($allowed_ext as $key => $value) {
            $allowed_ext[$key] = strtolower(trim($allowed_ext[$key]));
            if($allowed_ext[$key]==''){
                unset($allowed_ext[$key]);
            }
        }

        $id = JFactory::getApplication()->input->getInt('doccat');


        require_once JPATH_ADMINISTRATOR . '/components/com_edocman/libraries/rad/loader.php';
        JLoader::register('EDocmanModelList', JPATH_ROOT . '/components/com_edocman/model/list.php');

        $cats = EDocmanHelper::getChildrenCategories($id);

        if (count($cats)) {
            $parent_id = 1;
            foreach ($cats as $cat) {
                $cat = (int) $cat;
                $category = EDocmanHelper::getCategory($cat);

                $docs = OSModel::getInstance('List', 'EDocmanModel', array('ignore_session' => true, 'ignore_request' => true))
                    ->limitstart(0)
                    ->limit(0)
                    ->id($cat)
                    ->getData();



                $catcontroller = new DropfilesControllerCategory();

                $datas = array();
                $datas['jform']['extension'] = 'com_dropfiles';
                $datas['jform']['title'] = $category->title;
                $datas['jform']['alias'] = $category->alias.'-'.date('dmY-h-m-s',time());
                $datas['jform']['parent_id'] = $parent_id;
                $datas['jform']['published'] = 1;
                $datas['jform']['language'] = '*';
                $datas['jform']['metadata']['tags'] = '';

                //Set state value to retreive the correct table
                $model = $this->getModel('category');

                $model->setState('category.extension', 'category');

                foreach ($datas as $data => $val) {
                    JRequest::setVar($data, $val, 'POST');
                }

                if($catcontroller->save())
                {
                    $newId = $catcontroller->savedId;
                    $parent_id = $newId;

                    $edocmandir = JPATH_ROOT . '/edocman/';

                    $file_dir = dropfilesBase::getFilesPath($newId);
                    if(!file_exists($file_dir)){
                        JFolder::create($file_dir);
                        $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                        JFile::write($file_dir.'index.html', $data);
                        $data = 'deny from all';
                        JFile::write($file_dir.'.htaccess', $data);
                    }

                    $modelFiles = $this->getModel('files');
                    $user = JFactory::getUser();

                    foreach ($docs as $document) {

                        if ($document->document_url !='') {
                            $urlfile  = pathinfo($document->document_url);
                            $des = $document->short_description != '' ? $document->short_description : $document->description;
                            $id_file = $modelFiles->addFile(array(
                                'title' => $document->title,
                                'description' => $des,
                                'id_category' => $newId,
                                'file' => $document->document_url,
                                'ext' => $urlfile['extension'],
                                'size' => $this->remote_file_size($document->document_url),
                                'author' => $user->get('id')
                            ));

                        }else {

                            if(!in_array(strtolower(JFile::getExt($document->filename)),$allowed_ext)){
                                continue;
                            }

                            $newname = uniqid().'.'.strtolower(JFile::getExt($document->filename));

                            $docmanfile = $edocmandir . $document->filename;

                            if (file_exists($docmanfile)) {
                                JFile::copy($docmanfile, $file_dir . $newname);
                            }

                            //Insert new image into databse
                            $des = $document->short_description != '' ? $document->short_description : $document->description;
                            $id_file = $modelFiles->addFile(array(
                                'title' => $document->title,
                                'description' => $des,
                                'id_category' => $newId,
                                'file' => $newname,
                                'ext' => strtolower(JFile::getExt($document->filename)),
                                'size' => filesize($file_dir.$newname),
                                'author' => $user->get('id')
                            ));
                            if(!$id_file){
                                JFile::delete($file_dir.$newname);
                            }

                        }

                    }

                }
            }
        }

        $this->exit_status('Done');
    }
    /**
     *
     * Import docman files from selected category
     * @since version
     */
    public function docimport() {

        include_once JPATH_ADMINISTRATOR . '/components/com_dropfiles/controllers/category.php';

        $params = JComponentHelper::getParams('com_dropfiles');

        $allowed_ext = explode(',', $params->get('allowedext','7z,ace,bz2,dmg,gz,rar,tgz,zip,csv,doc,docx,html,key,keynote,odp,ods,odt,pages,pdf,pps,ppt,pptx,rtf,tex,txt,xls,xlsx,xml,bmp,exif,gif,ico,jpeg,jpg,png,psd,tif,tiff,aac,aif,aiff,alac,amr,au,cdda,flac,m3u,m4a,m4p,mid,mp3,mp4,mpa,ogg,pac,ra,wav,wma,3gp,asf,avi,flv,m4v,mkv,mov,mpeg,mpg,rm,swf,vob,wmv'));
        foreach ($allowed_ext as $key => $value) {
            $allowed_ext[$key] = strtolower(trim($allowed_ext[$key]));
            if($allowed_ext[$key]==''){
                unset($allowed_ext[$key]);
            }
        }

        $id = JFactory::getApplication()->input->getInt('doccat');
        $dbo = JFactory::getDbo();
        $query = 'SELECT GROUP_CONCAT(r.descendant_id ORDER BY r.level ASC SEPARATOR \'/\') as path ,r.level'
                .' FROM #__docman_category_relations as r INNER JOIN #__docman_categories as c  ON c.docman_category_id=r.ancestor_id  WHERE c.docman_category_id='.$id.' GROUP BY r.ancestor_id';
        $dbo->setQuery($query);

        $list = $dbo->loadObject();

        $cats = explode('/', $list->path);

        if (count($cats)) {
            $parent_id = 1;
            foreach ($cats as $catid) {
                $model = KObjectManager::getInstance()->getObject('com://admin/docman.model.documents')
                    ->enabled(1)
                    ->status('published')
                    ->category($catid)
                    ->limit(0)
                    ->sort('title')
                    ->direction('ASC')
                ;

                $documents = $model->fetch();

                $cat = KObjectManager::getInstance()->getObject('com://admin/docman.model.categories')->id($catid)->fetch();

                $catcontroller = new DropfilesControllerCategory();

                $datas = array();
                $datas['jform']['extension'] = 'com_dropfiles';
                $datas['jform']['title'] = $cat->title;
                $datas['jform']['alias'] = $cat->slug.'-'.date('dmY-h-m-s',time());
                $datas['jform']['parent_id'] = $parent_id;
                $datas['jform']['published'] = 1;
                $datas['jform']['language'] = '*';
                $datas['jform']['metadata']['tags'] = '';

                //Set state value to retreive the correct table
                $model = $this->getModel('category');

                $model->setState('category.extension', 'category');

                foreach ($datas as $data => $val) {
                    JRequest::setVar($data, $val, 'POST');
                }

                if($catcontroller->save())
                {
                    $newId = $catcontroller->savedId;
                    $parent_id = $newId;

                    $docmandir = JPATH_ROOT . '/joomlatools-files/docman-files/';

                    $file_dir = dropfilesBase::getFilesPath($newId);
                    if(!file_exists($file_dir)){
                        JFolder::create($file_dir);
                        $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                        JFile::write($file_dir.'index.html', $data);
                        $data = 'deny from all';
                        JFile::write($file_dir.'.htaccess', $data);
                    }

                    $modelFiles = $this->getModel('files');
                    $user = JFactory::getUser();

                    foreach ($documents as $document) {

                        if ($document->storage_type =='remote') {
                            $urlfile  = pathinfo($document->storage_path);
                            $id_file = $modelFiles->addFile(array(
                                'title' => $document->title,
                                'description' => $document->description,
                                'id_category' => $newId,
                                'file' => $document->storage_path,
                                'ext' => $urlfile['extension'],
                                'size' => $this->remote_file_size($document->storage_path),
                                'author' => $user->get('id')
                            ));

                        }else if($document->storage_type == 'file') {
                            if(!in_array(strtolower(JFile::getExt($document->storage_path)),$allowed_ext)){
                                continue;
                            }

                            $newname = uniqid().'.'.strtolower(JFile::getExt($document->storage_path));

                            $docmanfile = $docmandir . $document->storage_path;

                            if (file_exists($docmanfile)) {
                                JFile::copy($docmanfile, $file_dir . $newname);
                            }

                            //Insert new image into databse

                            $id_file = $modelFiles->addFile(array(
                                'title' => $document->title,
                                'description' => $document->description,
                                'id_category' => $newId,
                                'file' => $newname,
                                'ext' => strtolower(JFile::getExt($document->storage_path)),
                                'size' => filesize($file_dir.$newname),
                                'author' => $user->get('id')
                            ));
                            if(!$id_file){
                                JFile::delete($file_dir.$newname);
                            }

                        }

                    }

                }
            }
        }

        $this->exit_status('Done');

    }

    /**
     *
     * Get file size from a remote file
     * @param $url
     *
     * @return mixed|string
     *
     * @since version
     */
    protected function remote_file_size($url){

        $ch = curl_init($url);
        curl_setopt_array($ch, array(
            CURLOPT_RETURNTRANSFER  => 1,
            CURLOPT_FOLLOWLOCATION  => 1,
            CURLOPT_SSL_VERIFYPEER  => 0,
            CURLOPT_NOBODY          => 1,
        ));

        curl_setopt($ch, CURLOPT_NOBODY, 1);
        curl_exec($ch);

        $clen = curl_getinfo($ch, CURLINFO_CONTENT_LENGTH_DOWNLOAD);
        curl_close($ch);

        if (!$clen || ($clen == -1)) {
            return 'n/a';
        }

        return $clen;

    }
}