<?php
/**
 * Dropfiles
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.controllerform');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

class DropfilesControllerFiles extends JControllerForm
{

    private $allowed_ext = array('7z', 'ace', 'bz2', 'dmg', 'gz', 'rar', 'tgz', 'zip', 'csv', 'doc', 'docx', 'html', 'key', 'keynote', 'odp', 'ods', 'odt', 'pages', 'pdf', 'pps', 'ppt', 'pptx', 'rtf', 'tex', 'txt', 'xls', 'xlsx', 'xml', 'bmp', 'exif', 'gif', 'ico', 'jpeg', 'jpg', 'png', 'psd', 'tif', 'tiff', 'aac', 'aif', 'aiff', 'alac', 'amr', 'au', 'cdda', 'flac', 'm3u', 'm4a', 'm4p', 'mid', 'mp3', 'mp4', 'mpa', 'ogg', 'pac', 'ra', 'wav', 'wma', '3gp', 'asf', 'avi', 'flv', 'm4v', 'mkv', 'mov', 'mpeg', 'mpg', 'rm', 'swf', 'vob', 'wmv');

    public function __construct($config = array())
    {
        $params = JComponentHelper::getParams('com_dropfiles');
        $this->allowed_ext = explode(',', $params->get('allowedext', '7z,ace,bz2,dmg,gz,rar,tgz,zip,csv,doc,docx,html,key,keynote,odp,ods,odt,pages,pdf,pps,ppt,pptx,rtf,tex,txt,xls,xlsx,xml,bmp,exif,gif,ico,jpeg,jpg,png,psd,tif,tiff,aac,aif,aiff,alac,amr,au,cdda,flac,m3u,m4a,m4p,mid,mp3,mp4,mpa,ogg,pac,ra,wav,wma,3gp,asf,avi,flv,m4v,mkv,mov,mpeg,mpg,rm,swf,vob,wmv'));
        foreach ($this->allowed_ext as $key => $value) {
            $this->allowed_ext[$key] = strtolower(trim($this->allowed_ext[$key]));
            if ($this->allowed_ext[$key] == '') {
                unset($this->allowed_ext[$key]);
            }
        }
        parent::__construct($config);
    }

    public function upload()
    {


        // Access check.
        if (!$this->allowAdd()) {
            $this->exit_status(JText::_('JLIB_APPLICATION_ERROR_CREATE_RECORD_NOT_PERMITTED'));
        }

        $id_category = JFactory::getApplication()->input->getInt('id_category', 0);
        if ($id_category <= 0) {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_CATEGORY'));
        }

        if (strtolower($_SERVER['REQUEST_METHOD']) != 'post') {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_HTTP_RESPONSE'));
        }
        $app = JFactory::getApplication();
        $this->canEdit($id_category);

        $modelCat = $this->getModel('category');
        $category = $modelCat->getCategory($id_category);

        if ($category->type == 'googledrive') {

        } else {
            //todo: créer un répertoire spécial pour les categories
            $file_dir = dropfilesBase::getFilesPath($id_category);
            if (!file_exists($file_dir)) {
                JFolder::create($file_dir);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($file_dir . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($file_dir . '.htaccess', $data);
            }
        }

        if (array_key_exists('pic', $_FILES) && $_FILES['pic']['error'] == 0) {
            $pic = $_FILES['pic'];

            $pic['name'] = html_entity_decode($pic['name']);

            if (!in_array(strtolower(JFile::getExt($pic['name'])), $this->allowed_ext)) {
                $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FILE_EXTENSION'), array('allowed ' => $this->allowed_ext));
            }
            $user = JFactory::getUser();
            if ($category->type == 'googledrive') {
                $google = new dropfilesGoogle();

                $fileContent = file_get_contents($pic['tmp_name']);

                if (!$insertedFile = $google->uploadFile($pic['name'], $fileContent, $pic['type'], $category->cloud_id)) {
                    $this->exit_status($google->getLastError());
                }
                $modelGoogle = $this->getModel('googlefiles');
                $file_data = $google->getFileObj($insertedFile);
                $file_data->catid = $category->cloud_id;
                $file_data->author = $user->get('id');
                unset($file_data->id);
                $modelGoogle->addFile($file_data);
            } elseif ($category->type == 'dropbox') {
                $dropbox = new dropfilesDropbox();

                $result = $dropbox->uploadFile($pic['name'], $pic['tmp_name'], $pic['size'], $category->path);

                if ($result) {
                    $modelDropbox = $this->getModel('dropboxfiles');
                    $file_data = array();
                    $file_data['id'] = 0;
                    $file_data['title'] = JFile::stripExt($result['name']);
                    $file_data['file_id'] = $result['id'];
                    $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                    $file_data['size'] = $result['size'];
                    $file_data['catid'] = $category->cloud_id;
                    $file_data['path'] = $result['path_lower'];
                    $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                    $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                    $file_data['author'] = $user->get('id');
                    $modelDropbox->save($file_data);
                }

            } else {
                $newname = uniqid() . '.' . strtolower(JFile::getExt($pic['name']));
                if (!JFile::upload($pic['tmp_name'], $file_dir . $newname, false, true)) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                }

                //Insert new image into databse
                $model = $this->getModel();
                $id_file = $model->addFile(array(
                    'title' => JFile::stripExt($pic['name']),
                    'id_category' => $id_category,
                    'file' => $newname,
                    'ext' => strtolower(JFile::getExt($pic['name'])),
                    'size' => filesize($file_dir . $newname),
                    'author' => $user->get('id')
                ));
                if (!$id_file) {
                    JFile::delete($file_dir . $newname);
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_SAVE_TO_DB'));
                }
            }
            $params = JComponentHelper::getParams('com_dropfiles');

            $email_title = JText::_('COM_DROPEDITOR_DROPFILES_EMAIL_ADD_EVENT_TITLE');

            $email_body = $params->get('add_event_editor', DropfilesHelper::getHTMLEmail('file-added.html'));
            $email_body = str_replace('components/com_dropfiles/assets/images/icon-download.png', JUri::root() . 'components/com_dropfiles/assets/images/icon-download.png', $email_body);
            $email_body = str_replace('{category}', $category->title, $email_body);
            $email_body = str_replace('{website_url}', JUri::root(), $email_body);
            $email_body = str_replace('{file_name}', $pic['name'], $email_body);

            if ($params->get('add_event_subject', '') != '') {
                $email_title = $params->get('add_event_subject', '');
            }


            if ($params->get('file_owner', 0) == 1 && $params->get('add_event', 1) == 1) {
                $user = JFactory::getUser();
                $email_body = str_replace('{username}', $user->name, $email_body);
                DropfilesHelper::sendMail($user->email, $email_title, $email_body);
            }

            if ($params->get('category_owner', 0) == 1 && $params->get('add_event', 1) == 1) {
                $user = JFactory::getUser($category->created_user_id);
                $email_body = str_replace('{username}', $user->name, $email_body);
                DropfilesHelper::sendMail($user->email, $email_title, $email_body);
            }

            if ($params->get('add_event_additional_email', '') != '' && $params->get('add_event', 1) == 1) {
                $emails = explode(',', $params->get('add_event_additional_email', ''));
                if (!empty($emails)) {
                    foreach ($emails as $email) {
                        DropfilesHelper::sendMail($email, $email_title, $email_body);
                    }
                }
            }

            if ($params->get('notify_super_admin', 0) == 1 && $params->get('add_event', 1) == 1) {
                $users = DropfilesHelper::getSuperAdmins();

                if (count($users)) {
                    foreach ($users as $item) {

                        $user = JFactory::getUser($item->user_id);
                        $email_body = str_replace('{username}', $user->name, $email_body);
                        DropfilesHelper::sendMail($user->email, $email_title, $email_body);

                    }
                }
            }


            if ($app->isAdmin()) {
                $this->exit_status(true, array());
            } else {
                $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_UPLOAD_FILE_SUCCESS'), array());
            }

        }
        $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_UPLOAD_FILE_ERROR')); //todo : translate
    }

    public function version()
    {
        $id_file = JFactory::getApplication()->input->getString('id_file', null);
        $id_category = JFactory::getApplication()->input->getInt('id_category', 0);
        $ext = JFactory::getApplication()->input->getString('ext', '');
        if ($id_file === null) {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FILE'));
        }

        if (strtolower($_SERVER['REQUEST_METHOD']) != 'post') {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_HTTP_RESPONSE'));
        }

        $params = JComponentHelper::getParams('com_dropfiles');
        if (array_key_exists('pic', $_FILES) && $_FILES['pic']['error'] == 0) {
            $pic = $_FILES['pic'];
            if (!in_array(strtolower(JFile::getExt($pic['name'])), $this->allowed_ext)) {
                $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FILE_EXTENSION'), array('allowed ' => $this->allowed_ext));
            }

            if ($ext != strtolower(JFile::getExt($pic['name']))) {
                $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_ADD_VERSION_WRONG_EXT'), array());
            }
            $modelCat = $this->getModel('category');
            $category = $modelCat->getCategory($id_category);
            $modelFiles = $this->getModel();

            if($category->type=='googledrive'){
                $google = new dropfilesGoogle();
                $files = $google->listVersions($id_file);
            }elseif($category->type == 'dropbox') {
                $dropbox = new dropfilesDropbox();
                $files = $dropbox->displayDropboxVersionInfo($id_file);
            }else{
                $files = $modelFiles->getVersions($id_file);
            }

            if (count($files) >= $params->get('versioning_number', 10)) {
                $modelFiles->deleteOldestVersion($id_file, $category->id);
            }

            $this->canEdit($category->id);

            if ($category->type == 'googledrive') {
                $google = new dropfilesGoogle();

                $fileContent = file_get_contents($pic['tmp_name']);

                $google->saveFileInfos(array('id' => $id_file, 'newRevision' => true, 'title' => $pic['name'], 'data' => $fileContent, 'ext' => strtolower(JFile::getExt($pic['name']))), $category->cloud_id);
            } elseif ($category->type == 'dropbox') {
                $dropbox = new dropfilesDropbox();
                $version = $dropbox->saveDropboxVersion(array('newRevision' => true,
                    'old_file' => $id_file,
                    'new_file_name' => $pic['name'],
                    'new_file_size' => $pic['size'],
                    'new_tmp_name' => $pic['tmp_name']
                ));

            } else {
                $model = $this->getModel();
                $file = $model->getFile($id_file);

                if ($file->catid !== $category->id) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                }

                $version_dir = dropfilesBase::getVersionPath($id_category);
                if (!file_exists($version_dir)) {
                    JFolder::create($version_dir);
                    $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                    JFile::write($version_dir . 'index.html', $data);
                    $data = 'deny from all';
                    JFile::write($version_dir . '.htaccess', $data);
                }

                $newname = uniqid() . '.' . strtolower(JFile::getExt($pic['name']));

                if (JFile::move(dropfilesBase::getFilesPath($file->catid) . $file->file, $version_dir . $file->file) !== true) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                }

                if (!JFile::upload($pic['tmp_name'], dropfilesBase::getFilesPath($file->catid) . $newname, false, true)) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                }

                $modelF = $this->getModel('file');
                $table = $modelF->getTable();

                $updateData = array('id' => $file->id, 'file' => $newname, 'ext' => JFile::getExt($pic['name']), 'size' => filesize(dropfilesBase::getFilesPath($file->catid) . $newname));
                $version = JFactory::getApplication()->input->getString('version', null);
                if ($version !== null) {
                    $updateData['version'] = $version;
                }
                $table->save($updateData);

                $model->addVersion(array('id_file' => $file->id, 'file' => $file->file, 'ext' => $file->ext, 'size' => $file->size));
            }

            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_ADD_VERSION_SUCCESSFULLY'));
        }
    }

    public function import()
    {
        $user = JFactory::getUser();
        if (!$user->authorise('core.admin')) {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_PERMISSION'));
        }
        $id_category = JFactory::getApplication()->input->getInt('id_category', 0);
        if ($id_category <= 0) {
            $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_CATEGORY'));
        }
        if (!JSession::checkToken('get')) {
            $this->exit_status(JText::_('WRONG_TOKEN'));
        }

        $modelCat = $this->getModel('category');
        $category = $modelCat->getCategory($id_category);
        $this->canEdit($category->id);

        $params = JComponentHelper::getParams('com_dropfiles');
        $do = $params->get('import');
        if (!$do) {
            $this->exit_status('', array('noerror'));
        }
        //todo: créer un répertoire spécial pour les categories
        $file_dir = dropfilesBase::getFilesPath($id_category);
        if (!file_exists($file_dir)) {
            JFolder::create($file_dir);
            $data = '<html><body bgcolor="#FFFFFF"></body></html>';
            JFile::write($file_dir . 'index.html', $data);
            $data = 'deny from all';
            JFile::write($file_dir . '.htaccess', $data);
        }
        $files = JFactory::getApplication()->input->get('files', null, 'array');
        if (!empty($files)) {
            $count = 0;
            foreach ($files as $file) {
                $file = JPATH_ROOT . DIRECTORY_SEPARATOR . $file;
                if (strpos($file, '..') !== false) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FOLDER'));
                }
                if ((JPATH_ROOT != '') && strpos($file, JPath::clean(JPATH_ROOT)) !== 0) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FOLDER'));
                }
                if (!in_array(strtolower(JFile::getExt($file)), $this->allowed_ext)) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_WRONG_FILE_EXTENSION'), array('allowed ' => $this->allowed_ext));
                }
                if (!file_exists($file)) {
                    $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                }

                if ($category->type == 'googledrive') {
                    $google = new dropfilesGoogle();
                    if (!$google->uploadFile(JFile::getName($file), file_get_contents($file), '', $category->cloud_id)) {
                        $this->exit_status($google->getLastError());
                    }
                } else {
                    $newname = uniqid() . '.' . strtolower(JFile::getExt($file));
                    if (!JFile::copy($file, $file_dir . $newname)) {
                        $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_MOVE_FILE'));
                    }

                    //Insert new image into databse
                    $model = $this->getModel();
                    $user = JFactory::getUser();
                    $id_file = $model->addFile(array(
                        'title' => JFile::stripExt(JFile::getName($file)),
                        'id_category' => $id_category,
                        'file' => $newname,
                        'ext' => strtolower(JFile::getExt($file)),
                        'size' => filesize($file_dir . $newname),
                        'author' => $user->get('id')
                    ));
                    if (!$id_file) {
                        JFile::delete($file_dir . $newname);
                        $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_CANT_SAVE_TO_DB'));
                    }
                }
                $count++;
            }
            $this->exit_status(true, array('nb' => $count));
        }
        $this->exit_status(JText::_('COM_DROPEDITOR_DROPFILES_CTRL_FILES_UPLOAD_FILE_ERROR')); //todo : translate
    }

    public function movefile()
    {
        $input = JFactory::getApplication()->input;
        $id_category = $input->getInt('id_category', 0);
        $active_category_id = $input->getInt('active_category', 0);
        $id_file = $input->getString('id_file', null);

        if (!$this->allowEdit()) {
            $this->exit_status(JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
        }

        $model = $this->getModel();
        $modelFile = $this->getModel('file');
        $modelCategory = $this->getModel('category');
        $active_category = $modelCategory->getCategory($active_category_id);
        $target_category = $modelCategory->getCategory($id_category);
        if ($active_category->type == 'default' && $target_category->type == 'default') {

            $catpath_dest = dropfilesBase::getFilesPath($id_category);
            $file = $modelFile->getItem($id_file);
            $catpath_current = dropfilesBase::getFilesPath($file->catid);
            if ($model->moveCatFile($id_file, $id_category)) {
                // move file

                $file_current = $catpath_current . $file->file;
                $file_dest = $catpath_dest . $file->file;

                if (!file_exists($catpath_dest)) {
                    JFolder::create($catpath_dest);
                    $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                    JFile::write($catpath_dest . 'index.html', $data);
                    $data = 'deny from all';
                    JFile::write($catpath_dest . '.htaccess', $data);
                }

                if (is_file($file_current)) {
                    JFile::move($file_current, $file_dest);
                }


            }

        } else if ($active_category->type == 'googledrive' && $target_category->type == 'default') {

            $google = new dropfilesGoogle();
            $file = $google->download($id_file, $active_category->cloud_id, false);
            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . $file->ext;
            file_put_contents($catpath_dest . $newname, $file->datas);
            $user = JFactory::getUser();
            $model = $this->getModel();
            $new_file = $model->addFile(array(
                'title' => $file->title,
                'id_category' => $id_category,
                'file' => $newname,
                'ext' => $file->ext,
                'size' => $file->size,
                'author' => $user->get('id')
            ));

            if ($new_file) {
                $google->delete($id_file, $active_category->cloud_id);
            }

        } else if ($active_category->type == 'default' && $target_category->type == 'googledrive') {

            $google = new dropfilesGoogle();
            $catpath_dest = dropfilesBase::getFilesPath($id_category);
            $file = $modelFile->getItem($id_file);
            $modelFiles = $this->getModel('files');
            $catpath_current = dropfilesBase::getFilesPath($file->catid);
            $file_current = $catpath_current . $file->file;

            if (!$google->uploadFile($file->title . '.' . $file->ext, file_get_contents($file_current), '', $target_category->cloud_id)) {
                $this->exit_status($google->getLastError());
            }
            if ($modelFiles->removePicture($id_file)) {
                JFile::delete($file_current);
            }

        } else if ($active_category->type == 'googledrive' && $target_category->type == 'googledrive') {
            $google = new dropfilesGoogle();
            $file = $google->moveFile($id_file, $target_category->cloud_id);
            if ($file) {
                $user = JFactory::getUser();
                $modelGoogle = $this->getModel('googlefiles');
                $modelGoogle->deleteFile($id_file);
                $file_data = $google->getFileObj($file);
                $file_data->catid = $target_category->cloud_id;
                $file_data->author = $user->get('id');
                unset($file_data->id);
                $modelGoogle->addFile($file_data);
            }

        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'default') {

            $dropbox = new dropfilesDropbox();
            list($tem, $file) = $dropbox->downloadDropbox($id_file);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . JFile::getExt($file['name']);

            ob_start();
            header('Content-Disposition: attachment; filename="' . $file['name'] . '"');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');

            header('Content-Length: ' . (int)$file['size']);

            echo readfile($tem);
            unlink($tem);
            $data = ob_get_clean();
            file_put_contents($catpath_dest . $newname, $data);
            $user = JFactory::getUser();
            $model = $this->getModel();
            $new_file = $model->addFile(array(
                'title' => JFile::stripExt($file['name']),
                'id_category' => $id_category,
                'file' => $newname,
                'ext' => strtolower(JFile::getExt($file['name'])),
                'size' => $file['size'],
                'author' => $user->get('id')
            ));

            if ($new_file) {
                $modelDropbox = $this->getModel('dropboxfiles');
                if ($dropbox->deleteFileDropbox($id_file)) {
                    $modelDropbox->deleteFile($id_file);
                }
            }
        } elseif ($active_category->type == 'default' && $target_category->type == 'dropbox') {

            $file = $modelFile->getItem($id_file);
            $catpath_current = dropfilesBase::getFilesPath($file->catid);
            $file_current = $catpath_current . $file->file;
            $dropbox = new dropfilesDropbox();

            $result = $dropbox->uploadFile($file->title . '.' . $file->ext, $file_current, filesize($file_current), $target_category->path);

            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $file_data = array();
                $file_data['id'] = 0;
                $file_data['title'] = JFile::stripExt($result['name']);
                $file_data['file_id'] = $result['id'];
                $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                $file_data['size'] = $result['size'];
                $file_data['catid'] = $target_category->cloud_id;
                $file_data['path'] = $result['path_lower'];
                $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                $modelDropbox->save($file_data);
                $modelFiles = $this->getModel('files');
                if ($modelFiles->removePicture($id_file)) {
                    JFile::delete($file_current);
                }

            }
        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'dropbox') {

            $dropbox = new dropfilesDropbox();
            $file = $dropbox->getDropboxFileInfos($id_file);
            $result = $dropbox->moveFile($file['path_lower'], $target_category->path . '/' . strtolower($file['name']));
            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $modelDropbox->deleteFile($id_file);

                if ($result) {
                    $file_data = array();
                    $file_data['id'] = 0;
                    $file_data['title'] = JFile::stripExt($result['name']);
                    $file_data['file_id'] = $result['id'];
                    $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                    $file_data['size'] = $result['size'];
                    $file_data['catid'] = $target_category->cloud_id;
                    $file_data['path'] = $result['path_lower'];
                    $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                    $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                    $modelDropbox->save($file_data);
                }

            }
        } elseif ($active_category->type == 'googledrive' && $target_category->type == 'dropbox') {

            $google = new dropfilesGoogle();
            $file = $google->download($id_file, $active_category->cloud_id, false);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . $file->ext;
            file_put_contents($catpath_dest . $newname, $file->datas);

            $dropbox = new dropfilesDropbox();

            $file_current = $catpath_dest . $newname;

            $result = $dropbox->uploadFile($file->title . '.' . $file->ext, $file_current, filesize($file_current), $target_category->path);

            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $file_data = array();
                $file_data['id'] = 0;
                $file_data['title'] = JFile::stripExt($result['name']);
                $file_data['file_id'] = $result['id'];
                $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                $file_data['size'] = $result['size'];
                $file_data['catid'] = $target_category->cloud_id;
                $file_data['path'] = $result['path_lower'];
                $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                $modelDropbox->save($file_data);

                JFile::delete($file_current);

                if ($google->delete($id_file, $active_category->cloud_id)) {
                    $modelGoogle = $this->getModel('googlefiles');
                    $modelGoogle->deleteFile($id_file);
                }

            }
        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'googledrive') {

            $dropbox = new dropfilesDropbox();
            list($tem, $file) = $dropbox->downloadDropbox($id_file);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . JFile::getExt($file['name']);

            ob_start();
            header('Content-Disposition: attachment; filename="' . $file['name'] . '"');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');

            header('Content-Length: ' . (int)$file['size']);

            echo readfile($tem);
            unlink($tem);
            $data = ob_get_clean();
            $file_current = $catpath_dest . $newname;
            file_put_contents($file_current, $data);
            $google = new dropfilesGoogle();
            if ($insertedFile = $google->uploadFile($file['name'], file_get_contents($file_current), '', $target_category->cloud_id)) {
                $modelGoogle = $this->getModel('googlefiles');
                $user = JFactory::getUser();
                $file_data = $google->getFileObj($insertedFile);
                $file_data->catid = $target_category->cloud_id;
                $file_data->author = $user->get('id');
                unset($file_data->id);
                $modelGoogle->addFile($file_data);
                JFile::delete($file_current);

                $modelDropbox = $this->getModel('dropboxfiles');
                if ($dropbox->deleteFileDropbox($id_file)) {
                    $modelDropbox->deleteFile($id_file);
                }

            }
        }
        $this->exit_status(true, array('id_file' => $id_file));
        JFactory::getApplication()->close();
    }

    public function copyfile()
    {
        $input = JFactory::getApplication()->input;
        $id_category = $input->getInt('id_category', 0);
        $active_category_id = $input->getInt('active_category', 0);
        $id_file = $input->getString('id_file', null);

        if (!$this->allowEdit()) {
            $this->exit_status(JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
        }

        $model = $this->getModel();
        $modelFile = $this->getModel('file');
        $catpath_dest = dropfilesBase::getFilesPath($id_category);
        $file = $modelFile->getItem($id_file);
        $catpath_current = dropfilesBase::getFilesPath($file->catid);

        $modelCategory = $this->getModel('category');
        $active_category = $modelCategory->getCategory($active_category_id);
        $target_category = $modelCategory->getCategory($id_category);

        if ($active_category->type == 'default' && $target_category->type == 'default') {

            if ($file->catid != $id_category) {
                $data = array();
                $newname = uniqid() . '.' . strtolower(JFile::getExt($file->file));
                $user = JFactory::getUser();
                $data['file'] = $newname;
                $data['id_category'] = $id_category;
                $data['title'] = $file->title;
                $data['ext'] = $file->ext;
                $data['size'] = $file->size;
                $data['author'] = $user->get('id');
                if ($model->addFile($data)) {
                    // move file

                    $file_current = $catpath_current . $file->file;
                    $file_dest = $catpath_dest . $newname;

                    if (!file_exists($catpath_dest)) {
                        JFolder::create($catpath_dest);
                        $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                        JFile::write($catpath_dest . 'index.html', $data);
                        $data = 'deny from all';
                        JFile::write($catpath_dest . '.htaccess', $data);
                    }


                    if (is_file($file_current)) {
                        JFile::copy($file_current, $file_dest);
                    }


                }
            }
        } else if ($active_category->type == 'googledrive' && $target_category->type == 'default') {
            $google = new dropfilesGoogle();
            $file = $google->download($id_file, $active_category->cloud_id, false);
            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . $file->ext;
            file_put_contents($catpath_dest . $newname, $file->datas);

            $model = $this->getModel();
            $user = JFactory::getUser();
            $new_file = $model->addFile(array(
                'title' => $file->title,
                'id_category' => $id_category,
                'file' => $newname,
                'ext' => $file->ext,
                'size' => $file->size,
                'author' => $user->get('id')
            ));

        } else if ($active_category->type == 'default' && $target_category->type == 'googledrive') {
            $google = new dropfilesGoogle();
            $file = $modelFile->getItem($id_file);
            $catpath_current = dropfilesBase::getFilesPath($file->catid);
            $file_current = $catpath_current . $file->file;

            if (!$google->uploadFile($file->title . '.' . $file->ext, file_get_contents($file_current), '', $target_category->cloud_id)) {
                $this->exit_status($google->getLastError());
            }
        } else if ($active_category->type == 'googledrive' && $target_category->type == 'googledrive') {
            $google = new dropfilesGoogle();
            $google->copyFile($id_file, $target_category->cloud_id);
        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'default') {

            $dropbox = new dropfilesDropbox();
            list($tem, $file) = $dropbox->downloadDropbox($id_file);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . JFile::getExt($file['name']);

            ob_start();
            header('Content-Disposition: attachment; filename="' . $file['name'] . '"');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');

            header('Content-Length: ' . (int)$file['size']);

            echo readfile($tem);
            unlink($tem);
            $data = ob_get_clean();
            file_put_contents($catpath_dest . $newname, $data);
            $user = JFactory::getUser();
            $model = $this->getModel();
            $new_file = $model->addFile(array(
                'title' => JFile::stripExt($file['name']),
                'id_category' => $id_category,
                'file' => $newname,
                'ext' => strtolower(JFile::getExt($file['name'])),
                'size' => $file['size'],
                'author' => $user->get('id')
            ));

        } elseif ($active_category->type == 'default' && $target_category->type == 'dropbox') {

            $file = $modelFile->getItem($id_file);
            $catpath_current = dropfilesBase::getFilesPath($file->catid);
            $file_current = $catpath_current . $file->file;
            $dropbox = new dropfilesDropbox();


            $result = $dropbox->uploadFile($file->title . '.' . $file->ext, $file_current, filesize($file_current), $target_category->path);

            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $file_data = array();
                $file_data['id'] = 0;
                $file_data['title'] = JFile::stripExt($result['name']);
                $file_data['file_id'] = $result['id'];
                $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                $file_data['size'] = $result['size'];
                $file_data['catid'] = $target_category->cloud_id;
                $file_data['path'] = $result['path_lower'];
                $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                $modelDropbox->save($file_data);
            }


        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'dropbox') {
            $dropbox = new dropfilesDropbox();
            $file = $dropbox->getDropboxFileInfos($id_file);

            $result = $dropbox->copyFileDropbox($file['path_lower'], $target_category->path . '/' . strtolower($file['name']));
            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $file_data = array();
                $file_data['id'] = 0;
                $file_data['title'] = JFile::stripExt($result['name']);
                $file_data['file_id'] = $result['id'];
                $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                $file_data['size'] = $result['size'];
                $file_data['catid'] = $target_category->cloud_id;
                $file_data['path'] = $result['path_lower'];
                $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                $modelDropbox->save($file_data);
            }
        } elseif ($active_category->type == 'googledrive' && $target_category->type == 'dropbox') {
            $google = new dropfilesGoogle();
            $file = $google->download($id_file, $active_category->cloud_id, false);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . $file->ext;
            file_put_contents($catpath_dest . $newname, $file->datas);

            $dropbox = new dropfilesDropbox();

            $file_current = $catpath_dest . $newname;

            $result = $dropbox->uploadFile($file->title . '.' . $file->ext, $file_current, filesize($file_current), $target_category->path);

            if ($result) {
                $modelDropbox = $this->getModel('dropboxfiles');
                $file_data = array();
                $file_data['id'] = 0;
                $file_data['title'] = JFile::stripExt($result['name']);
                $file_data['file_id'] = $result['id'];
                $file_data['ext'] = strtolower(JFile::getExt($result['name']));
                $file_data['size'] = $result['size'];
                $file_data['catid'] = $target_category->cloud_id;
                $file_data['path'] = $result['path_lower'];
                $file_data['created_time'] = date('Y-m-d H:i:s', strtotime($result['client_modified']));
                $file_data['modified_time'] = date('Y-m-d H:i:s', strtotime($result['server_modified']));
                $modelDropbox->save($file_data);

                JFile::delete($file_current);
            }

        } elseif ($active_category->type == 'dropbox' && $target_category->type == 'googledrive') {

            $dropbox = new dropfilesDropbox();
            list($tem, $file) = $dropbox->downloadDropbox($id_file);

            $catpath_dest = dropfilesBase::getFilesPath($id_category);

            if (!file_exists($catpath_dest)) {
                JFolder::create($catpath_dest);
                $data = '<html><body bgcolor="#FFFFFF"></body></html>';
                JFile::write($catpath_dest . 'index.html', $data);
                $data = 'deny from all';
                JFile::write($catpath_dest . '.htaccess', $data);
            }

            $newname = uniqid() . '.' . JFile::getExt($file['name']);

            ob_start();
            header('Content-Disposition: attachment; filename="' . $file['name'] . '"');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');

            header('Content-Length: ' . (int)$file['size']);

            echo readfile($tem);
            unlink($tem);
            $data = ob_get_clean();
            $file_current = $catpath_dest . $newname;
            file_put_contents($file_current, $data);
            $google = new dropfilesGoogle();
            if ($insertedFile = $google->uploadFile($file['name'], file_get_contents($file_current), '', $target_category->cloud_id)) {
                $modelGoogle = $this->getModel('googlefiles');
                $user = JFactory::getUser();
                $file_data = $google->getFileObj($insertedFile);
                $file_data->catid = $target_category->cloud_id;
                $file_data->author = $user->get('id');
                unset($file_data->id);
                $modelGoogle->addFile($file_data);
                JFile::delete($file_current);
            }
        }

        $this->exit_status(true);
        JFactory::getApplication()->close();
    }

    /**
     * Delete a file
     */
    public function delete()
    {
        $return = false;
        $id_file = JRequest::getString('id_file', 0);
        $id_cat = JRequest::getInt('id_cat', 0);

        if (!$this->allowEdit()) {
            $this->exit_status(JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
        }

        $modelC = $this->getModel('category');
        $category = $modelC->getDropfileCategory($id_cat);
        $category2 = $modelC->getCategory($id_cat);
        $author_user_id = 0;
        if ($category) {
            switch ($category->type) {
                case 'googledrive';
                    $google = new dropfilesGoogle();
                    $modelGoogle = $this->getModel('googlefiles');
                    $file = $modelGoogle->getFile($id_file);
                    $author_user_id = $file->author;
                    if ($google->delete($id_file, $category->cloud_id)) {
                        $return = true;
                        $modelGoogle->deleteFile($id_file);
                    }
                    break;
                case 'dropbox':

                    $dropbox = new dropfilesDropbox();
                    $modelDropbox = $this->getModel('dropboxfiles');
                    $file = $modelDropbox->getFile($id_file);
                    $author_user_id = $file->author;
                    if ($dropbox->deleteFileDropbox($id_file)) {
                        $return = true;
                        $modelDropbox->deleteFile($id_file);
                    }
                    break;

                default :
                    $model = $this->getModel();
                    $file = $model->getFile($id_file);
                    $author_user_id = $file->author;
                    if ($file !== false) {
                        $this->canEdit($file->catid);
                    }
                    $file_dir = dropfilesBase::getFilesPath($file->catid);
                    if (file_exists($file_dir . $file->file)) {
                        JFile::delete($file_dir . $file->file);
                    }
                    if ($model->removePicture($file->id)) {
                        $return = true;
                    } else {
                        $return = false;
                    }
                    break;
            }
            $params = JComponentHelper::getParams('com_dropfiles');

            $email_title = JText::_('COM_DROPEDITOR_DROPFILES_EMAIL_DELETE_EVENT_TITLE');

            if ($params->get('delete_event_subject', '') != '') {
                $email_title = $params->get('delete_event_subject', '');
            }

            $email_body = $params->get('delete_event_editor', DropfilesHelper::getHTMLEmail('file-deleted.html'));
            $email_body = str_replace('components/com_dropfiles/assets/images/icon-download.png', JUri::root() . 'components/com_dropfiles/assets/images/icon-download.png', $email_body);
            $email_body = str_replace('{category}', $category2->title, $email_body);
            $email_body = str_replace('{website_url}', JUri::root(), $email_body);
            $email_body = str_replace('{file_name}', $file->title, $email_body);

            if ($params->get('file_owner', 0) == 1 && $params->get('delete_event', 1) == 1) {
                $user = JFactory::getUser($author_user_id);
                $email_body = str_replace('{username}', $user->name, $email_body);
                DropfilesHelper::sendMail($user->email, $email_title, $email_body);
            }

            if ($params->get('category_owner', 0) == 1 && $params->get('delete_event', 1) == 1) {
                $user = JFactory::getUser($category2->created_user_id);
                $email_body = str_replace('{username}', $user->name, $email_body);
                DropfilesHelper::sendMail($user->email, $email_title, $email_body);
            }

            if ($params->get('delete_event_additional_email', '') != '' && $params->get('delete_event', 1) == 1) {
                $emails = explode(',', $params->get('delete_event_additional_email', ''));
                if (!empty($emails)) {
                    foreach ($emails as $email) {
                        DropfilesHelper::sendMail($email, $email_title, $email_body);
                    }
                }
            }

            if ($params->get('notify_super_admin', 0) == 1 && $params->get('delete_event', 1) == 1) {
                $users = DropfilesHelper::getSuperAdmins();

                if (count($users)) {
                    foreach ($users as $item) {

                        $user = JFactory::getUser($item->user_id);
                        $email_body = str_replace('{username}', $user->name, $email_body);
                        DropfilesHelper::sendMail($user->email, $email_title, $email_body);

                    }
                }
            }


        }
        echo json_encode($return);

        JFactory::getApplication()->close();
    }

    /**
     * Reorder category
     */
    public function reorder()
    {
        $files = JRequest::getString('order', null);
        $idcat = JFactory::getApplication()->input->getInt('idcat', false);
        $modelCat = $this->getModel('category');
        $category = $modelCat->getCategory($idcat);
        $this->canEdit($category->id);

        $files = json_decode($files);

        if ($category->type === 'googledrive') {
            //$google = new dropfilesGoogle();
            //$return = $google->reorder($files,$category->cloud_id);
            $modelGoogle = $this->getModel('googlefiles');
            foreach ($files as $key => $file) {
                $f = $modelGoogle->getFile($file);
                $filesok = true;
                if ($f->catid != $category->cloud_id) {
                    $filesok = false;
                    break;
                }
            }
            if ($filesok) {
                if ($modelGoogle->reorder($files)) {
                    $return = true;
                } else {
                    $return = false;
                }
            } else {
                $return = false;
            }

        } elseif ($category->type === 'dropbox') {
            $modelDropbox = $this->getModel('dropboxfiles');
            foreach ($files as $key => $file) {
                $f = $modelDropbox->getFile($file);
                $filesok = true;
                if ($f->catid != $category->cloud_id) {
                    $filesok = false;
                    break;
                }
            }
            if ($filesok) {
                if ($modelDropbox->reorder($files)) {
                    $return = true;
                } else {
                    $return = false;
                }
            } else {
                $return = false;
            }
        } else {
            $model = $this->getModel();
            foreach ($files as $key => $file) {
                $f = $model->getFile($file);
                $filesok = true;
                if ($f->catid != $category->id) {
                    $filesok = false;
                    break;
                }
            }

            if ($filesok) {
                if ($model->reorder($files)) {
                    $return = true;
                } else {
                    $return = false;
                }
            } else {
                $return = false;
            }
        }
        echo json_encode($return);
        JFactory::getApplication()->close();
    }


    /**
     * Return a json response
     * @param $status
     * @param array $datas array of datas to return with the json string
     *
     */
    private function exit_status($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }

    /**
     * Check if the current user has permission on the current gallery
     * @param type $id_gallery
     */
    private function canEdit($id_category)
    {
        $model = $this->getModel('category');
        $canDo = DropfilesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $category = $model->getItem($id_category);
                if ($category->created_user_id != JFactory::getUser()->id) {
                    $this->exit_status('not permitted');
                }
            } else {
                $this->exit_status('not permitted');
            }
        }
    }

    /**
     * Add a remote file by url
     * @since version
     */
    public function addremoteurl()
    {


        // Access check.
        if (!$this->allowAdd()) {
            $this->exit_status(JText::_('JLIB_APPLICATION_ERROR_CREATE_RECORD_NOT_PERMITTED'));
        }

        $input = JFactory::getApplication()->input;
        $id_category = $input->getInt('id_category');
        if ($id_category <= 0) {
            $this->exit_status(JText::_('Wrong Category'));
        }

        $remote_title = $input->getString('remote_title');

        $remote_url = $input->getString('remote_url');
        $remote_type = $input->getString('remote_type');
        if ($remote_title == '') {
            $this->exit_status(JText::_('Enter title'));
        } else if ($remote_type == '') {
            $this->exit_status(JText::_('Enter type'));
        } else if ($remote_url == '') {
            $this->exit_status(JText::_('Enter url'));
        } else {
            if (!preg_match("(http://|https://)", $remote_url)) {
                $this->exit_status(JText::_($remote_url . " is not a valid URL"));
            }
        }

        //Insert new image into databse
        $model = $this->getModel();
        $user = JFactory::getUser();
        $id_file = $model->addFile(array(
            'title' => $remote_title,
            'id_category' => (int)$id_category,
            'file' => $remote_url,
            'ext' => $remote_type,
            'size' => $this->remote_file_size($remote_url),
            'author' => $user->get('id')
        ), true);

        if (!$id_file) {
            $this->exit_status(JText::_('Cant save to database'));
        }

        $this->exit_status(true, array('id_file' => $id_file, 'name' => $remote_url));

    }

    /**
     * Get file size from a remote file
     * @param $url
     *
     * @return mixed|string
     *
     * @since version
     */
    protected function remote_file_size($url)
    {

        $ch = curl_init($url);
        curl_setopt_array($ch, array(
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_FOLLOWLOCATION => 1,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_NOBODY => 1,
        ));

        curl_setopt($ch, CURLOPT_NOBODY, 1);
        curl_exec($ch);

        $clen = curl_getinfo($ch, CURLINFO_CONTENT_LENGTH_DOWNLOAD);
        curl_close($ch);

        if (!$clen || ($clen == -1)) {
            return 'n/a';
        }

        return $clen;

    }
}