<?php
/** 
 * Dropfiles
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') or die;

//jimport('joomla.application.component.modeladmin');
//jimport('joomla.access.access');

require_once(JPATH_ROOT.DIRECTORY_SEPARATOR.'administrator'.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_categories'.DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'categories.php');

class DropfilesModelCategories extends CategoriesModelCategories
{   
    
    protected  $canDo;
    
    public function __construct($config = array()) {
        parent::__construct($config);
        $app = JFactory::getApplication();
        $app->setUserState('com_categories.categories.filter.extension', 'com_dropfiles');
        $app->setUserState('list.limit',100000000);
    }

    
    public function populateState($ordering = null, $direction = null) {
        parent::populateState($ordering, $direction);
        $this->setState('list.start', 0);
        $this->setState('filter.extension', 'com_dropfiles');
        $this->state->set('list.limit', 100000000);
    }
    
    public function getListQuery() {
        $db = $this->getDbo();       
        $this->setState('filter.access',null); //don't want to use Joomla access

        $query = $db->getQuery(true);
        

        // Select the required fields from the table.
        $query->select(
                $this->getState(
                        'list.select',
                        'a.id, a.title, a.alias, a.note, a.published, a.access' .
                        ', a.checked_out, a.checked_out_time, a.created_user_id' .
                        ', a.path, a.parent_id, a.level, a.lft, a.rgt' .
                        ', a.language'
                )
        );
        $query->from('#__categories AS a');

        // Join over the language
        $query->select('l.title AS language_title')
                ->join('LEFT', $db->quoteName('#__languages') . ' AS l ON l.lang_code = a.language');

        // Join over the users for the checked out user.
        $query->select('uc.name AS editor')
                ->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

        //Join over the dropfiles categories
        $query->select('d.type AS type')
                ->join('LEFT', '#__dropfiles AS d ON d.id=a.id');
        
        // Filter by extension
        if ($extension = $this->getState('filter.extension'))
        {
                $query->where('a.extension = ' . $db->quote($extension));
        }

        // Filter on the level.
        if ($level = $this->getState('filter.level'))
        {
                $query->where('a.level <= ' . (int) $level);
        }
        
        // Filter by published state
        $published = $this->getState('filter.published');
        if (is_numeric($published))
        {
                $query->where('a.published = ' . (int) $published);
        }
        elseif ($published === '')
        {
                $query->where('(a.published IN (0, 1))');
        }

        $catid = $this->getState('category.id',null);
        if($catid!==null){
            $subQuery = "SELECT rgt,lft FROM #__categories WHERE id=".(int)$catid." AND extension='com_dropfiles'";
            $db->setQuery($subQuery);
            if(!$db->query()){
                return false;
            }
            $parent = $db->loadObject();
            $recursive = $this->getState('category.recursive',null);
            if($recursive){
                $query->where('a.rgt<= '.(int)$parent->rgt);
                $query->where('a.lft> '.(int)$parent->lft);
            }else{
                $query->where('a.parent_id = '.(int)$catid);
            }
        }
        
        $query->select('COUNT(f.id) as files');
        $query->join('LEFT', '#__dropfiles_files AS f ON a.id = f.catid');
//        $query->where('a.id IS NOT null');
        $query->group('a.id');

        if($this->getState('category.frontcategories',false)==false){
            $canDo = DropfilesHelper::getActions();
            if(($canDo->get('core.edit.own') && !$canDo->get('core.edit'))||(!$canDo->get('core.edit.own') && !$canDo->get('core.edit'))){
                $query->where('created_user_id='.(int)JFactory::getUser()->id);
            }
        }        
        $query->order('a.lft ASC');

        return $query;
        
    }
    /*Sync Page Task*/
    public function getAllCat(){
        // Get a db connection.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true);$query
            ->select("*")
            ->from('#__categories AS c, #__dropfiles AS d')
            ->where($db->quoteName('c.id')." = ".$db->quoteName('d.id')." AND d.type=".$db->quote('googledrive'));

        $db->setQuery($query);
        $results = $db->loadObjectList();
        return $results;
    }
    /*Sync Page Task*/
    public function getAllDropboxCat(){
        // Get a db connection.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true);$query
            ->select("*")
            ->from('#__categories AS c, #__dropfiles AS d')
            ->where($db->quoteName('c.id')." = ".$db->quoteName('d.id')." AND d.type=".$db->quote('dropbox'));

        $db->setQuery($query);
        $results = $db->loadObjectList();
        return $results;
    }
    public function getIdLastOnCategories(){
        // Get a db connection.
        $db = $this->getDbo();

        $query = $db->getQuery(true);$query
            ->select("id")
            ->from('#__categories AS c')
            ->where('extension="com_dropfiles"')
            ->order('created_time DESC, id DESC');

        $db->setQuery($query);
        $results = $db->loadResult();
        return $results;

    }
    public function ArrayCloudIdDropfiles(){
        
        $db=$this->getDbo();
        $query =$db->getQuery(true);
        $query->select('cloud_id')
            ->from($db->quoteName('#__dropfiles'))
            ->where($db->quoteName('type')." = ".$db->quote('googledrive'));
        $db->setQuery($query);
        $result=$db->loadColumn(); 
        return $result;
      
    }
    public function getOneCatByCloudId($cloud_id){
        // Get a db connection.
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select("*")
            ->from('#__categories AS c, #__dropfiles AS d')
            ->where($db->quoteName('c.id')." = ".$db->quoteName('d.id')." AND d.cloud_id=".$db->quote($cloud_id));

        $db->setQuery($query);
        $results = $db->loadObject();
        return $results;
    }

    public function createDropboxOnDropfiles($id,$type,$cloud_id,$path){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        // Insert columns.
        $columns = array('id', 'type', 'cloud_id', 'path');
        // Insert values.
        $values = array($db->quote($id),$db->quote($type),$db->quote($cloud_id),$db->quote($path));
        $query->insert($db->quoteName('#__dropfiles'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        $db->execute();
    }

    public function getOneCatByLocalId($local_id){
        // Get a db connection.
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select("*")
            ->from('#__categories AS c, #__dropfiles AS d')
            ->where($db->quoteName('c.id')." = ".$db->quoteName('d.id')." AND c.id=".$db->quote($local_id));

        $db->setQuery($query);
        $results = $db->loadObject();
        return $results;
    }

    public function updatePathDropboxById($id,$path){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        $query="UPDATE #__dropfiles SET path=".$db->quote($path)." WHERE id=".$db->quote($id);
        $db->setQuery($query);
        return $db->execute();
    }

    public function createOnDropfiles($id,$type,$cloud_id){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        // Insert columns.
        $columns = array('id', 'type', 'cloud_id');
        // Insert values.
        $values = array($id,$db->quote($type),$db->quote($cloud_id));
        $query->insert($db->quoteName('#__dropfiles'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        $db->execute();
    }
    public function createOnCategories($title,$parent_id,$level){
        $basePath = JPATH_ADMINISTRATOR . '/components/com_categories';
        require_once $basePath . '/models/category.php';
        $config = array( 'table_path' => $basePath . '/tables');
        $catmodel = new CategoriesModelCategory( $config);

        $catData = array(
            'id' => 0, 'parent_id' => $parent_id, 'level' =>$level, 'extension' => 'com_dropfiles',
            'title' => $title, 'alias' =>JString::increment($title),
            'published' => 1, 'language' => '*','associations'=>array()
        );
        $result= $catmodel->save( $catData);
        if($result) {
             $newCatId = $this->getIdLastOnCategories();
             return $newCatId;
        }
        return $result;
    }
    public function deleteOnDropfiles($id){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        $conditions = array(
            $db->quoteName('cloud_id') . ' = ' . $db->quote($id)
        );
        $query->delete($db->quoteName('#__dropfiles'));
        $query->where($conditions);
        $db->setQuery($query);
        return $db->execute();

    }
    public function deleteOnCategories($id){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        $conditions = array(
            $db->quoteName('id') . ' = ' . $db->quote($id)
        );
        $query->delete($db->quoteName('#__categories'));
        $query->where($conditions);
        $db->setQuery($query);
        return $db->execute();
    }
    public function updateTitleById($id,$title){
        // Get a db connection.
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        $query="UPDATE #__categories SET title=".$db->quote($title)." WHERE id=".$db->quote($id);
        $db->setQuery($query);
        return $db->execute();
    }
    
    public function getAllTagsFiles(){
        $dbo = $this->getDbo();
        $query = "SELECT catid, GROUP_CONCAT(file_tags SEPARATOR ',') as cat_tags FROM #__dropfiles_files WHERE file_tags !='' GROUP BY catid";
        $dbo->setQuery($query);$dbo->query();
        $list_tags = $dbo->loadAssocList('catid');
       
        $catTags = array();
        foreach ($list_tags as $key => $value) {                       
            $allTags1 = explode(',', $value["cat_tags"]);
            $allTags2 = array_values(array_unique($allTags1));
            $catTags[$key] = $allTags2;
        }        
     
        $params = JComponentHelper::getParams('com_dropfiles');         
        if( $params->get('google_credentials','') ) {   
            JLoader::register('DropfilesGoogle', JPATH_ADMINISTRATOR.'/components/com_dropfiles/classes/dropfilesGoogle.php');	
            $google = new dropfilesGoogle();            
            $googleCats = $this->getAllCat();
            if(!empty($googleCats)) {
                $gCatTags = $google->getAllTagsFileOnGoogle($googleCats);
                if(is_array($gCatTags)) {
                    $catTags = array_merge($catTags,$gCatTags);
                }
            }
        }
            
        return $catTags;       
    }
}