<?php
/**
 * Dropfiles
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.access.access');
jimport('joomla.application.component.modellist');


class DropfilesModelFrontfiles extends JModelList
{


    /**
     * Constructor.
     *
     * @param    array    An optional associative array of configuration settings.
     * @see        JController
     * @since    1.6
     */
    public function __construct($config = array())
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = array(
                'ordering', 'a.ordering'//,
            );
        }

        parent::__construct($config);
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @return    void
     * @since    1.6
     */
    protected function populateState($ordering = 'ordering', $direction = 'ASC')
    {
        $app = JFactory::getApplication();

        // List state information
        $this->setState('list.limit', 1000);

        $this->setState('list.start', 0);

        $this->setState('list.ordering', 'a.ordering');

        $this->setState('list.direction', 'ASC');

        $this->setState('filter.access', true);

        $catid = JRequest::getInt('id', 0);

        $this->setState('filter.category_id', $catid);
    }

    /**
     * Get the master query for retrieving a list of filess subject to the model state.
     *
     * @return    JDatabaseQuery
     * @since    1.6
     */
    function getListQuery()
    {
        // Create a new query object.
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
            $this->getState(
                'list.select',
                'a.id, a.title, a.file, ' . // a.alias, a.title_alias, a.introtext, ' .
                'a.ext, ' .
                'a.size, ' .
                'a.hits, ' .
                'a.description, ' .
                'a.version, ' .
                'a.canview, ' .
                'a.catid, a.created_time,a.modified_time,a.custom_icon'
            )
        );

        $query->from('#__dropfiles_files AS a');

        // Join over the categories.
        $query->select('c.title AS category_title, c.path AS category_route, c.access AS category_access, c.alias AS category_alias');
        $query->join('LEFT', '#__categories AS c ON c.id = a.catid');

        // Join over the categories to get parent category titles
        $query->select('parent.title as parent_title, parent.id as parent_id, parent.path as parent_route, parent.alias as parent_alias');
        $query->join('LEFT', '#__categories as parent ON parent.id = c.parent_id');

        // Join to check for category published state in parent categories up the tree
//		$query->select('c.published, CASE WHEN badcats.id is null THEN c.published ELSE 0 END AS parents_published');
        $query->select('c.published, c.published AS parents_published');
        $subquery = 'SELECT cat.id as id FROM #__categories AS cat JOIN #__categories AS parent ';
        $subquery .= 'ON cat.lft BETWEEN parent.lft AND parent.rgt ';
        $subquery .= 'WHERE parent.extension = ' . $db->quote('com_dropfiles');

        $query->join('LEFT OUTER', '(' . $subquery . ') AS badcats ON badcats.id = c.id');

        // Filter by access level.
        if ($access = $this->getState('filter.access')) {
            $user = JFactory::getUser();
            $groups = implode(',', $user->getAuthorisedViewLevels());
//			$query->where('a.access IN ('.$groups.')');
            $query->where('c.access IN (' . $groups . ')');
        }

        // Filter by a single or group of categories
        $categoryId = $this->getState('filter.category_id');

        if (is_numeric($categoryId)) {
            $type = $this->getState('filter.category_id.include', true) ? '= ' : '<> ';

            // Add subcategory check
            $includeSubcategories = $this->getState('filter.subcategories', false);
            $categoryEquals = 'a.catid ' . $type . (int)$categoryId;

            if ($includeSubcategories) {
                $levels = (int)$this->getState('filter.max_category_levels', '1');
                // Create a subquery for the subcategory list
                $subQuery = $db->getQuery(true);
                $subQuery->select('sub.id');
                $subQuery->from('#__categories as sub');
                $subQuery->join('INNER', '#__categories as this ON sub.lft > this.lft AND sub.rgt < this.rgt');
                $subQuery->where('this.id = ' . (int)$categoryId);
                if ($levels >= 0) {
                    $subQuery->where('sub.level <= this.level + ' . $levels);
                }

                // Add the subquery to the main query
                $query->where('(' . $categoryEquals . ' OR a.catid IN (' . $subQuery->__toString() . '))');
            } else {
                $query->where($categoryEquals);
            }
        } elseif (is_array($categoryId) && (count($categoryId) > 0)) {
            JArrayHelper::toInteger($categoryId);
            $categoryId = implode(',', $categoryId);
            if (!empty($categoryId)) {
                $type = $this->getState('filter.category_id.include', true) ? 'IN' : 'NOT IN';
                $query->where('a.catid ' . $type . ' (' . $categoryId . ')');
            }
        }
        // Filter by publish dates.
        $nullDate = $db->quote($db->getNullDate());
        $date = JFactory::getDate();

        $nowDate = $db->quote($date->toSql());

        $query->where('(a.publish = ' . $nullDate . ' OR a.publish <= ' . $nowDate . ')');
        $query->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');
        $query->where('a.state = 1');
        // Filter by language
        if ($this->getState('filter.language')) {
            $query->where('a.language in (' . $db->quote(JFactory::getLanguage()->getTag()) . ',' . $db->quote('*') . ')');
        }

        // Add the list ordering clause.
        $query->order($this->getState('list.ordering', 'a.ordering') . ' ' . $this->getState('list.direction', 'ASC'));

        $query->group('a.id, a.title, a.catid,  a.state, badcats.id, c.title, c.path, c.access, c.alias, parent.title, parent.id, parent.path, parent.alias, c.published, c.lft, parent.lft, c.id');

        return $query;
    }

    /**
     * Method to get a list of filess.
     *
     * Overriden to inject convert the attribs field into a JParameter object.
     *
     * @return    mixed    An array of objects on success, false on failure.
     * @since    1.6
     */
    public function getItems()
    {
        return parent::getItems();
    }

    public function getStart()
    {
        return 0;
    }


}