<?php
/**
 * Dropeditor
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropeditor
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */
//-- No direct access
defined('_JEXEC') || die('=;)');


jimport('joomla.plugin.plugin');

/**
 * Content Plugin.
 *
 * @package    Dropeditor
 * @subpackage Plugin
 */
class plgContentDropeditor extends JPlugin {

    /**
     * Example before display content method
     *
     * Method is called by the view and the results are imploded and displayed in a placeholder
     *
     * @param  string  $context     The context for the content passed to the plugin.
     * @param  object  &$article    The content object.  Note $article->text is also available
     * @param  object  &$params     The content params
     * @param  int     $limitstart  The 'page' number
     *
     * @return string
     */
    public function onContentPrepare($context, &$article, &$params, $limitstart) {

        $app = JFactory::getApplication();
        $preview = $app->input->get('dropeditor_preview');
        $previewTitle = $app->input->get('dropeditor_preview_title', '' , 'RAW');
        $previewData = $app->input->get('dropeditor_preview_data', '' , 'RAW');

        if(trim($article->text) == '') return true;

        // Load preview article
        if ($preview) {
            $article->title = $previewTitle;
            $article->text = $previewData;
        }

        $document = JFactory::getDocument();
        $document->addStyleSheet(JURI::root() . 'plugins/editors/dropeditor/dropeditor.css');
        $document->addStyleSheet(JURI::root() . 'media/dropeditor/customstyles.css');
        $document->addStyleSheet(JURI::root() . 'media/dropeditor/titlestyles.css');

        // Render videos and sounds from img tags
        $text = preg_replace_callback('@<img[^>]*?data\-dropmedia="true".*?>@', array($this,'replaceMedia'),$article->text);
	    // Setup plyr if needed
        $plyr_loaded = false;
        if($text !== $article->text && $text !== null){
		    $article->text = $text;
		    $document->addScriptDeclaration('jQuery(document).ready(function ($) {plyr.setup();});');
		    $plyr_loaded = true;
	    }
	    if (strpos($article->text, 'dropmediaLocalVideo') && !$plyr_loaded) {
        	$this->enqueuePlyr();
		    $document->addScriptDeclaration('jQuery(document).ready(function ($) {plyr.setup();});');
		    $plyr_loaded = true;
	    }

        $componentParams = JComponentHelper::getParams('com_dropeditor');
        $defaultColor = $componentParams->get('default_color','#333333');  
        $contentCss = "";
        $contentCss .= "ol.ol-circles > li:before { background-color: ". $defaultColor.";}" . "\n";
        $contentCss .= "ol.ol-boxes > li:before { background-color: ". $defaultColor.";}" . "\n";

		libxml_use_internal_errors(true);
		$dom = new DOMDocument();
		$dom->loadHTML( mb_convert_encoding($article->text, 'HTML-ENTITIES', 'UTF-8'));

		try {
			# remove <!DOCTYPE
            if ($dom->doctype) $dom->removeChild($dom->doctype);
			# remove <html><body></body></html>
			if(!empty($dom->firstChild) && !empty($dom->firstChild->firstChild) ) {
				$dom->replaceChild($dom->firstChild->firstChild, $dom->firstChild);
			}
		}catch(Exception $e) {
			//do nothing
		}

		$docElement= $dom->documentElement ? $dom->documentElement : $dom->firstChild;
		if ($docElement) {
            if ($docElement->nodeName != 'body') {
                if (!empty($docElement->nextSibling) && !empty($docElement->nextSibling->firstChild)) {
                    $docElement = $docElement->nextSibling->firstChild;
                }
            }

            if ($docElement->nodeName != 'body') return true;

            $inlineStyles = $docElement->getElementsByTagName('style');
            $tagspans     = $docElement->getElementsByTagName('span');
        }

		JLoader::register('DropeditorHelper', JPATH_ADMINISTRATOR.'/components/com_dropeditor/helpers/dropeditor.php');
		$url = JPATH_ADMINISTRATOR.'/components/com_dropeditor/views/ggfonts/tmpl/fonts.json';
		$fonts_json = DropeditorHelper::gf_download_font_list($url);

		$fonts_object = json_decode($fonts_json);
		$fonts_list = array();
		foreach ($fonts_object->items as $font){
			$fonts_list[$font->family] = strtolower($font->family);
		}

		if($tagspans->length >0){
			$fo = array();
			foreach ($tagspans as $span) {
				$style = $span->getAttribute('style');
				$a = explode(';', $style);
				foreach($a as $b){
					if(strpos($b, 'font-family') !== false){
						$c = explode(':', $b);
						if(in_array($c[1], $fo) == false){
							$fo[] = $c[1];
						}
					}
				}
			}

			$list = '';
			foreach ($fo as $f){
				if(in_array(strtolower(trim($f,' ')), $fonts_list)){
					$key = array_search(strtolower(trim($f,' ')), $fonts_list);
					$list .= $key .'|';
				}
			}
			if(!empty($list)){
				$document->addStyleSheet('//fonts.googleapis.com/css?family='.$list);
			}

		}

		if($inlineStyles->length >0 ) {
			$styleToRemove = array();
			foreach ($inlineStyles as $inlineStyle) {
				$styleToRemove[] = $inlineStyle;
				$contentCss .= $this->domInnerHTML($inlineStyle);
			}
			foreach($styleToRemove as $node)
			{
			  $node->parentNode->removeChild($node);
			}
			$article->text = $dom->saveHTML($dom->documentElement);
		}

        $document->addStyleDeclaration($contentCss);
        
        return true;
    }

    public function domInnerHTML(DOMNode $element) 
    { 
        $innerHTML = ""; 
        $children  = $element->childNodes;

        foreach ($children as $child) 
        { 
            $innerHTML .= $element->ownerDocument->saveHTML($child);
        }

        return $innerHTML; 
    }

    // Render video from img tag
	public function replaceMedia($match) {
    	// Include plyr
		$this->enqueuePlyr();

		preg_match('@src=\"([^"]+)\"@', $match[0], $vid_src);
		preg_match('@data-host=\"([^"]+)\"@', $match[0], $vid_type);
		preg_match('@data-video-title=\"([^"]+)\"@', $match[0], $vid_title);
		preg_match('@data-video-id=\"([^"]+)\"@', $match[0], $vid_id);
		preg_match('@data-max-width=\"([^"]+)\"@', $match[0], $vid_max_width);
		preg_match('@data-load-type=\"([^"]+)\"@', $match[0], $vid_load_type);
		preg_match('@data-direct-link=\"([^"]+)\"@', $match[0], $vid_direct_link);
		preg_match('@data-local-file=\"([^"]+)\"@', $match[0], $vid_local_file);
		$vid_max_width = (int) $vid_max_width[1] ? (int) $vid_max_width[1] : 'none';
		$vid_max_width = ($vid_max_width == 'none') ? '100%' : $vid_max_width.'px';

		if ($vid_load_type[1] == 'content') {
			$content = '<div class="videoWrapper" style="max-width: '. $vid_max_width .'">';
			if ($vid_type[1] == 'youtube' || $vid_type[1] == 'vimeo') {
				$content .= '<div data-type="'. $vid_type[1] .'" data-video-id="'. $vid_id[1] .'"></div>';
			} else if ($vid_type[1] == 'video') {
				$content .= '<video controls src="'. $vid_local_file[1] .'"></video>';
			} else if ($vid_type[1] == 'sound') {
				$content .= '<audio controls src="'. $vid_local_file[1] .'"></audio>';
			}
			$content .= '</div>';

			return $content;
		} elseif ($vid_load_type[1] == 'popup') {
			$this->enqueuePlyr(true);

			if ($vid_type[1] == 'youtube' || $vid_type[1] == 'vimeo') {
				$content = '<div class="dropmediaVideo" style="max-width: '. $vid_max_width .'; min-width: 200px">';
				$content .= '<a href="'. $vid_direct_link[1] .'" title="'. $vid_title[1] .'" data-title="'. $vid_title[1] .'" data-host="'. $vid_type[1] .'" data-video-id="'. $vid_id[1] .'">';
				$content .= '<img src="'. $vid_src[1] .'" title="'. $vid_title[1] .'" />';
				$content .= '</a>';
				$content .= '</div>';

				return $content;
			} elseif ($vid_type[1] == 'video' || $vid_type[1] == 'sound') {
				$content = '<div class="dropmediaVideo" style="max-width: '. $vid_max_width .'; min-width: 200px">';
				$content .= '<a href="#" data-host="'. $vid_type[1] .'" title="'. $vid_title[1] .'" data-title="'. $vid_title[1] .'" data-local-src="'. $vid_local_file[1] .'">';
				$content .= '<img src="'. $vid_src[1] .'" title="'. $vid_title[1] .'" />';
				$content .= '</a>';
				$content .= '</div>';

				return $content;
			}
		}

		return $match[0];
	}

	public function enqueuePlyr($colorbox = false) {
		$doc = JFactory::getDocument();
		$doc->addStyleSheet(JURI::base('true').'/components/com_dropeditor/assets/css/plyr.css');
		$doc->addScript(JURI::base('true').'/components/com_dropeditor/assets/js/plyr.js');
		// Add need files for popup video
		if ($colorbox) {
			$doc->addStyleSheet(JURI::base('true').'/components/com_dropeditor/assets/css/plyr.colorbox.css');
			$doc->addScript(JURI::base('true').'/components/com_dropeditor/assets/js/jquery.colorbox.min.js');
			$doc->addScript(JURI::base('true').'/components/com_dropeditor/assets/js/plyr.colorbox.init.js');
		}

		return true;
	}

    public function onContentAfterSave($context, $article, $isNew)
    {
        return true;
		// Check we are handling the frontend edit form.
        if ($context != 'com_content.article')
	{
            return true;
	}
                
        jimport( 'joomla.filesystem.folder' );
        jimport( 'joomla.filesystem.file' );
            
            $folder = JPATH_SITE.DIRECTORY_SEPARATOR.'plugins'.DIRECTORY_SEPARATOR.'editors'.DIRECTORY_SEPARATOR.'dropeditor'.DIRECTORY_SEPARATOR;
            $file = $folder.'custom.css';
            
            JLoader::register('lessc', JPATH_ADMINISTRATOR.'/components/com_dropeditor/classes/lessphp.php');
            JLoader::register('csstidy', JPATH_ADMINISTRATOR.'/components/com_dropeditor/classes/csstidy/class.csstidy.php');

            $less = new lessc;  
            $customCss = '';
         
            
           preg_match('#<style(.*?)class="style-bul(.*?)>(.*?)</style>#s',$article->introtext, $matches);
             
            try {
                $customCss = $less->compile($matches[3]);
            } catch (Exception $exc) {
                return false;
            }
	    
            $csstidy = new csstidy();
            $csstidy->parse($customCss);
            $less->setFormatter('compressed');
            try {
                $content = $less->compile($csstidy->print->plain());
            } catch (Exception $exc) {
                return false;
            }
            
            if(JFile::write($file, $content)){                
                return true;
            }
    }
}
