﻿/** 
 * DropEditor
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package DropEditor
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

( function() {
	function getListElement( editor, listTag ) {
		var range;
		try {
			range = editor.getSelection().getRanges()[ 0 ];
		} catch ( e ) {
			return null;
		}

		range.shrink( CKEDITOR.SHRINK_TEXT );
		return editor.elementPath( range.getCommonAncestor() ).contains( listTag, 1 );
	}

	var listItem = function( node ) {
			return node.type == CKEDITOR.NODE_ELEMENT && node.is( 'li' );
		};

	var mapListStyle = {
		'a': 'lower-alpha',
		'A': 'upper-alpha',
		'i': 'lower-roman',
		'I': 'upper-roman',
		'1': 'decimal',
		'disc': 'disc',
		'circle': 'circle',
		'square': 'square'
	};

	function listStyle( editor, startupPage ) {
		var lang = editor.lang.customlist;
		if ( startupPage == 'bulletedListStyle' ) {
			return {
				title: lang.bulletedTitle,
				minWidth: 300,
				minHeight: 50,
				contents: [
					{
					id: 'info',
					accessKey: 'I',
					elements: [
						{
						type: 'select',
						label: lang.type,
						id: 'type',
						align: 'center',
						style: 'width:150px',
						items: [
							[ lang.notset, '' ],
							[ lang.pushpin, 'drop-pushpin' ],
							[ lang.cog, 'drop-cog' ],
							[ lang.flag, 'drop-flag' ],
                                                        [ lang.star, 'drop-star' ],
                                                        [ lang.checkmark, 'drop-checkmark' ],
							[ lang.minus, 'drop-minus' ],
							[ lang.plus, 'drop-plus' ],
                                                        [ lang.play, 'drop-play' ],
                                                        [ lang.arrowRight, 'drop-arrow-right' ],
							[ lang.checkboxChecked, 'drop-checkbox-checked' ],
							[ lang.radioChecked, 'drop-radio-checked' ],
                                                        [ lang.radioUnchecked, 'drop-radio-unchecked' ]
                                                        
							],                                               
                                                onChange: function( api ) {
                                                       
                                                     if(this.getValue() != "") {
                                                        document = CKEDITOR.dom.document
                                                        var preview = document.getElementById( 'iconPreview' );
                                                        preview.removeAttribute( 'class' );
                                                        preview.addClass( 'drop-bullet ' + this.getValue() );
                                                     }
                                                   
                                                },
						setup: function( element ) {
                                                        var value = "";
                                                        if(element.hasClass("drop-circle")) {
                                                            value = "drop-circle" ;
                                                        }else if(element.hasClass("drop-disc")) {
                                                            value = "drop-disc" ;
                                                        }else if(element.hasClass("drop-square")) {
                                                            value = "drop-square" ;
                                                        }
							
							this.setValue( value );
						},
						commit: function( element ) {
							var value = this.getValue();
							if ( value ) {
                                                            	element.removeAttribute( 'class' );
								element.addClass( 'drop-bullet ' + value );
                                                        } else
								element.removeAttribute( 'class' );
						}
					},
                                        {
                                                    type: 'html',                                                    
                                                    id: 'preview',
                                                    align: 'center',                                                    
                                                    html: '<ul id="iconPreview" class="drop-bullet"><li></li></ul>'
                                        }
                                    ]
				}
				],
				onShow: function() {
					var editor = this.getParentEditor(),
						element = getListElement( editor, 'ul' );

					element && this.setupContent( element );
				},
				onOk: function() {
					var editor = this.getParentEditor(),
						element = getListElement( editor, 'ul' );

					element && this.commitContent( element );
				}
			};
		} else if ( startupPage == 'numberedListStyle' ) {

			var listStyleOptions = [
				[ lang.notset, '' ],
				[ lang.lowerRoman, 'lower-roman' ],
				[ lang.upperRoman, 'upper-roman' ],
				[ lang.lowerAlpha, 'lower-alpha' ],
				[ lang.upperAlpha, 'upper-alpha' ],
				[ lang.decimal, 'decimal' ],
                                [ lang.olCircle, 'ol-circles' ],
                                [ lang.olBox, 'ol-boxes' ]
				];

			if ( !CKEDITOR.env.ie || CKEDITOR.env.version > 7 ) {
				listStyleOptions.concat( [
					[ lang.armenian, 'armenian' ],
					[ lang.decimalLeadingZero, 'decimal-leading-zero' ],
					[ lang.georgian, 'georgian' ],
					[ lang.lowerGreek, 'lower-greek' ]
					] );
			}

			return {
				title: lang.numberedTitle,
				minWidth: 300,
				minHeight: 50,
				contents: [
					{
					id: 'info',
					accessKey: 'I',
					elements: [
						{
						type: 'hbox',
						widths: [ '25%', '75%' ],
						children: [
							{
							label: lang.start,
							type: 'text',
							id: 'start',
							validate: CKEDITOR.dialog.validate.integer( lang.validateStartNumber ),
							setup: function( element ) {
								// List item start number dominates.
								var value = element.getFirst( listItem ).getAttribute( 'value' ) || element.getAttribute( 'start' ) || 1;
								value && this.setValue( value );
							},
							commit: function( element ) {
								var firstItem = element.getFirst( listItem );
								var oldStart = firstItem.getAttribute( 'value' ) || element.getAttribute( 'start' ) || 1;

								// Force start number on list root.
								element.getFirst( listItem ).removeAttribute( 'value' );
								var val = parseInt( this.getValue(), 10 );
								if ( isNaN( val ) )
									element.removeAttribute( 'start' );
								else
									element.setAttribute( 'start', val );

								// Update consequent list item numbering.
								var nextItem = firstItem,
									conseq = oldStart,
									startNumber = isNaN( val ) ? 1 : val;
								while ( ( nextItem = nextItem.getNext( listItem ) ) && conseq++ ) {
									if ( nextItem.getAttribute( 'value' ) == conseq )
										nextItem.setAttribute( 'value', startNumber + conseq - oldStart );
								}
							}
						},
							{
							type: 'select',
							label: lang.type,
							id: 'type',
							style: 'width: 100%;',
							items: listStyleOptions,
                                                        onChange: function( api ) {
                                                       
                                                            if(this.getValue() !== "") {
                                                               document = CKEDITOR.dom.document
                                                               var preview = document.getElementById( 'iconPreview' );
                                                               var start =  1;
                                                                preview.setStyle( 'counter-reset', 'li-counter '+ (start-1)  );
                                                               preview.removeAttribute( 'class' );
                                                               preview.setAttribute( 'type', this.getValue() );
                                                               preview.addClass(  this.getValue() );
                                                            }
                                                   
                                                        },
							setup: function( element ) {
								var value = element.getStyle( 'list-style-type' ) || mapListStyle[ element.getAttribute( 'type' ) ] || element.getAttribute( 'type' ) || '';

								this.setValue( value );
							},
							commit: function( element ) {
								var value = this.getValue();
                                                                element.removeAttribute( 'class' );                                                                
								if ( value ) {
                                                                    	var start = element.getFirst( listItem ).getAttribute( 'value' ) || element.getAttribute( 'start' ) || 1;
									element.setStyle( 'counter-reset', 'li-counter '+ (start-1)  );
                                                                        element.setAttribute( 'type', value );
                                                                        element.addClass(value);
                                                                }
								else {
									element.removeStyle( 'list-style-type' );
                                                                }
							}
						}
						]
					},
                                         {
                                                    type: 'html',                                                    
                                                    id: 'preview',
                                                    align: 'center',                                                    
                                                    html: '<ol id="iconPreview" class=""><li>Item preview</li></ol>'
                                        }
                                    ]
				}
				],
				onShow: function() {
					var editor = this.getParentEditor(),
						element = getListElement( editor, 'ol' );

					element && this.setupContent( element );
				},
				onOk: function() {
					var editor = this.getParentEditor(),
						element = getListElement( editor, 'ol' );

					element && this.commitContent( element );
				}
			};
		}
	}

	CKEDITOR.dialog.add( 'numberedListStyle', function( editor ) {
		return listStyle( editor, 'numberedListStyle' );
	} );

	CKEDITOR.dialog.add( 'bulletedListStyle', function( editor ) {
		return listStyle( editor, 'bulletedListStyle' );
	} );
} )();
