/**
 * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
 * For licensing, see LICENSE.md or http://ckeditor.com/license
 */

/**
 * @fileOverview Summary commands.
 */

( function() {


    function summaryCommand( editor ) {
        this.editor = editor;
    }

    summaryCommand.prototype = {
        exec: function( editor ) {
            var selection = editor.getSelection(),
                enterMode = editor.config.enterMode;

            if ( !selection )
                return;
            var element =  selection.getStartElement();
            var toc = new CKEDITOR.dom.element.createFromHtml(  '<div class="tableOfContent">' + buildTableOfContent(editor) + '</div>' );
            toc.insertBefore( element );

        }

    };

    function updateSummaryCommand( editor ) {
        this.editor = editor;
    }
    updateSummaryCommand.prototype = {
        exec: function( editor ) {
            var selection = editor.getSelection();
            var element =  selection.getStartElement();
            var toc  = null;
            while (element) {
                var name = element.getName();
                if (name == 'div' && element.hasClass("tableOfContent")) {
                    toc = element;
                    break;
                }

                element = element.getParent();
            }
            toc.setHtml( buildTableOfContent(editor) );
        }
    };

    function buildTableOfContent(editor) {
        var myTree = (editor.document.find('h1,h2,h3'));
        var myTreehtml = '<ul class="tableOfContent">';

        for (var i = 0; i < myTree.count(); i++) {
            var cNode = myTree.getItem(i);
            cTag = cNode.$.tagName;
            cLevel = parseInt(cTag.replace('H', ''));
            if(cNode.$.textContent == '') continue;

            anchorID = getAnchorID(cNode,'toc');

            if (i < myTree.count() - 1) {
                nNode = myTree.getItem(i + 1);
                nTag = nNode.$.tagName;
                nLevel = parseInt(nTag.replace('H', ''));
            } else {
                nLevel = cLevel;
            }

            myTreehtml += '<li class="toc-level'+cLevel+'">';
            myTreehtml += '<a href="javascript:void(0);" onclick="location.hash=\'#'+anchorID+'\';" >' + cNode.$.textContent + '</a>';
            if (cLevel != nLevel) {
                if (nLevel > cLevel) {
                    for (var j = 0; j < (nLevel - cLevel - 1); j++) {
                        myTreehtml += '<ul class="tableOfContent"><li class="toc-level'+(cLevel+j+1)+'">';
                    }
                    myTreehtml += '<ul class="tableOfContent">';
                } else {
                    // The next item is shallower.
                    myTreehtml += '</li>';
                    for (var j = 0; j < (cLevel - nLevel); j++) {
                        myTreehtml += '</ul></li>';
                    }
                }
            } else {
                myTreehtml += '</li>';
            }

        }
        myTreehtml += '</ul>';
        return myTreehtml;
    }
    verboseIdCache = {};
    function getAnchorID(heading, prefix) {
        var candidateId =  heading.$.textContent.replace(/[^a-z0-9]/ig, ' ').replace(/\s+/g, '-').toLowerCase();
        if (verboseIdCache[candidateId]) {
            var j = 2;

            while(verboseIdCache[candidateId + j]) {
                j++;
            }
            candidateId = candidateId + '-' + j;

        }
        verboseIdCache[candidateId] = true;
        heading.$.id = prefix + '-' + candidateId;

        return prefix + '-' + candidateId;
    }

    CKEDITOR.plugins.add( 'summary', {
        icons: 'summary', // %REMOVE_LINE_CORE%
        lang: 'af,ar,bg,bn,bs,ca,cs,cy,da,de,el,en,en-au,en-ca,en-gb,eo,es,et,eu,fa,fi,fo,fr,fr-ca,gl,gu,he,hi,hr,hu,id,is,it,ja,ka,km,ko,ku,lt,lv,mk,mn,ms,nb,nl,no,pl,pt,pt-br,ro,ru,si,sk,sl,sq,sr,sr-latn,sv,th,tr,tt,ug,uk,vi,zh,zh-cn', // %REMOVE_LINE_CORE%
        init: function( editor ) {

            var tableOfContent = new summaryCommand( editor  );
            editor.addCommand( 'tableOfContent', tableOfContent );

            if ( editor.ui.addButton ) {
                editor.ui.addButton( 'Summary', {
                    label: editor.lang.summary.title,
                    command: 'tableOfContent',
                    toolbar: 'insert,10'
                } );
            }


            var updateSummary = new updateSummaryCommand( editor  );
            editor.addCommand( 'updateSummary', updateSummary );

            //Register map group;
            editor.addMenuGroup( "summary", 100 );

            editor.addMenuItems( {
                updateSummary: {
                    label: editor.lang.summary.updateSummary,
                    group: 'summary',
                    command: 'updateSummary'
                }
            } );

            editor.contextMenu.addListener( function( element, selection ) {
                if ( !element || element.isReadOnly() )
                    return null;

                while ( element ) {
                    var name = element.getName();
                    if ( name == 'div' &&  element.hasClass("tableOfContent") )
                        return { updateSummary: CKEDITOR.TRISTATE_OFF };

                    element = element.getParent();
                }
                return null;
            } );

            //editor.on( 'dirChanged', onDirChanged );
        }
    } );
} )();