<?php
/** 
 * Dropeditor
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropeditor
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


defined('_JEXEC') or die;

/**
 * @package		Joomla.Administrator
 * @subpackage	com_messages
 * @since		1.6
 */
class DropeditorHelper
{
    
        /**
	 * @var    JObject  A cache for the available actions.
	 * @since  1.6
	 */
	protected static $actions;
                
        
	/**
	 * Configure the Linkbar.
	 *
	 * @param	string	The name of the active view.
	 *
	 * @return	void
	 * @since	1.6
	 */

	public static function addSubmenu($vName)
	{
		if( JFactory::getApplication()->isAdmin() ) {
			JSubMenuHelper::addEntry(
				JText::_('COM_DROPEDITOR_PROFILES'),
				'index.php?option=com_dropeditor&view=profiles',
				$vName == 'profiles'
			);

			JSubMenuHelper::addEntry(
				JText::_('COM_DROPEDITOR_STYLES'),
				'index.php?option=com_dropeditor&view=format',
				$vName == 'format'
			);
		}
	}
        
    
	/**
	 * Gets a list of the actions that can be performed.
	 *
	 * @return  JObject
	 *
	 * @since   1.6
	 * @todo    Refactor to work with notes
	 */
	public static function getActions()
	{
		if (empty(self::$actions))
		{
			$user = JFactory::getUser();
			self::$actions = new JObject;

			$actions = JAccess::getActions('com_dropeditor');

			foreach ($actions as $action)
			{
				self::$actions->set($action->name, $user->authorise($action->name, 'com_dropeditor'));
			}
		}

		return self::$actions;
	}

    public static function writeTitlesOption($titleOptions) {
        if(count($titleOptions)>0) {
            $datas = array();
            $format_tags  = array();
            $titleStyles ='';

            $format_tags[] = 'p';
            $data = new stdClass();
            $data->element = 'p';
            $data->name = 'Paragraph';
            $data->attributes =  array('class'=> '');
            $datas['format_p'] = $data;

            foreach ($titleOptions as $titleObj) {
	            if( $titleObj->element=='h1'){
                     $format_tags[]= 'heading1';
	            }else {
                  $format_tags[]= $titleObj->element;
	            }

	            $data = new stdClass();
	            $data->name = $titleObj->title;
	            $data->element = $titleObj->element;
	            $data->attributes =  array('class'=> 'drop_'.$titleObj->element);

	            $titleStyles .= '.drop_'.$titleObj->element.' {';
	            if($titleObj->css != "" && $titleObj->css != "{}") {
		            $cssObj  = json_decode($titleObj->css);

		            if(isset($cssObj->cell_font_family) ) {
			            $titleStyles .= "font-family:" . $cssObj->cell_font_family .";";
		            }
		            if(isset($cssObj->cell_font_size) ) {
			            $titleStyles .= "font-size:" . $cssObj->cell_font_size . 'px;';
		            }
					if(isset($cssObj->cell_font_color) ) {
		            	$titleStyles .= "color:" . $cssObj->cell_font_color .";";
					}
					if(isset($cssObj->cell_font_bold) ) {
					    $titleStyles .= "font-weight: bold;";
					}
					if(isset($cssObj->cell_font_underline) ) {
					    $titleStyles .= "text-decoration: underline;";
					}
					if(isset($cssObj->cell_font_italic) ) {
					    $titleStyles .= "font-style: italic;";
					}
					if(isset($cssObj->cell_align) ) {
					    $titleStyles .= "text-align:" . $cssObj->cell_align .";";
					}
					if(isset($cssObj->line_height) ) {
					    $titleStyles .= "line-height:" . $cssObj->line_height."px;";
					}
					if(isset($cssObj->margin) ) {
					    $titleStyles .= "margin:" . $cssObj->margin."px;";
					}
					if(isset($cssObj->padding) ) {
					    $titleStyles .= "padding:" . $cssObj->padding . "px;";
					}
					if(isset($cssObj->cell_border_top) ) {
				        $titleStyles .= "border-top:" . $cssObj->cell_border_top.";";
					}
					if(isset($cssObj->cell_border_right) ) {
					    $titleStyles .= "border-right:" . $cssObj->cell_border_right.";";
					}
					if(isset($cssObj->cell_border_bottom) ) {
					    $titleStyles .= "border-bottom:" . $cssObj->cell_border_bottom.";";
					}
					if(isset($cssObj->cell_border_left) ) {
					    $titleStyles .= "border-left:" . $cssObj->cell_border_left.";";
					}
					if(isset($cssObj->border) ) {
					    $titleStyles .= "border:" . $cssObj->border .";";
					}
		            if(isset($cssObj->custom_css) &&  $cssObj->custom_css != "") {
			            $titleStyles .= $cssObj->custom_css;
		            }
	            }

	            $titleStyles .= '}' . "\n";

	            if($titleObj->element=='h1')      {
		            $datas['format_heading1'] = $data;
	            }else {
		            $datas['format_'. $titleObj->element] = $data;
	            }
            }

            $datas['format_tags'] = implode(';', $format_tags) ;
	        $registry = new JRegistry;
            $registry->loadArray($datas);
            $config = $registry->toString();

            jimport('joomla.filesystem.path');
            jimport('joomla.filesystem.file');

            $file = JPATH_SITE . '/media/dropeditor/config.json';
            if (!JFile::write($file, $config))
            {
                    JFactory::getApplication()->enqueueMessage(JText::_('COM_DROPEDITOR_ERROR_WRITE_CONFIG_FAILED'), 'error');
                    return;
            }

            $file = JPATH_SITE . '/media/dropeditor/titlestyles.css';
            if (!JFile::write($file, $titleStyles))
            {
                    JFactory::getApplication()->enqueueMessage(JText::_('COM_DROPEDITOR_ERROR_WRITE_CONFIG_FAILED'), 'error');

                    return;
            }

            return true;
        }
    }
	
    public static function writeCustomStyle($customStyles) {
	    if(count($customStyles)>0) {
            $datas = array();
            $cssStyles =  '';
            $registry = new JRegistry;

		    $data = new stdClass();
            $data->name = 'Paragraph';
            $data->element = 'p';
            $data->attributes = array('class'=> '');
            $registry->loadArray($data);
            $datas[] =  $registry->toString();

            foreach ($customStyles as $customStyle) {
	            $data = new stdClass();
	            $data->name = $customStyle->title;
	            $data->element = 'div';
                $data->attributes = array('class'=> $customStyle->name);
	            if($customStyle->css != "" && $customStyle->css != "{}") {
		            $cssStyles .= '.'.$customStyle->name.' {';
                    $cssStyles .= $customStyle->css ;
                    $cssStyles .= '}';
	            }
	            $registry->loadArray($data);
                $datas[] =  $registry->toString();
            }

            jimport('joomla.filesystem.path');
            jimport('joomla.filesystem.file');

            $styles = " CKEDITOR.stylesSet.add( 'dropeditor', [ " . implode(",", $datas) . "]);";
            $file = JPATH_SITE . '/media/dropeditor/dropeditor_styles.js';
            if (!JFile::write($file, $styles))
            {
                    JFactory::getApplication()->enqueueMessage(JText::_('COM_DROPEDITOR_ERROR_WRITE_CONFIG_FAILED'), 'error');

                    return;
            }

		    $file = JPATH_SITE . '/media/dropeditor/customstyles.css';
            if (!JFile::write($file, $cssStyles))
            {
                    JFactory::getApplication()->enqueueMessage(JText::_('COM_DROPEDITOR_ERROR_WRITE_CONFIG_FAILED'), 'error');

                    return;
            }
	    }
    }

    /**
     * Load global file language
     */
    public static function loadLanguage(){
        $lang = JFactory::getLanguage();
        $lang->load('com_dropeditor',JPATH_ADMINISTRATOR.'/components/com_dropeditor',null,true);
        $lang->load('com_dropeditor.override',JPATH_ADMINISTRATOR.'/components/com_dropeditor',null,true);
        $lang->load('com_dropeditor.sys',JPATH_ADMINISTRATOR.'/components/com_dropeditor',null,true); // If sys fil exists
    }
        
    public static function gf_download_font_list($url) {
        $fonts_json = null;
        $fonts_json = @file_get_contents($url);
        return $fonts_json;
    }

    public static function getVideoInfo($vid, $type) {
    	$vInfo = array();
    	switch ($type) {
		    case 'youtube':
			    // Get title
			    $url = "https://www.youtube.com/watch?v=".$vid;
			    $html = @file_get_contents($url);

			    $doc = new DOMDocument();
			    $doc->strictErrorChecking = false;
			    $doc->recover=true;
			    $doc->preserveWhiteSpace = FALSE;
			    @$doc->loadHTML(mb_convert_encoding("<html><body>".$html."</body></html>",'HTML-ENTITIES', 'UTF-8'));
			    $title_div = $doc->getElementById('eow-title');
			    if (!$title_div) return false;

			    $vInfo['title'] = trim($title_div->nodeValue);
			    $vInfo['id'] = $vid;
			    $vInfo['type'] = 'youtube';
			    $vInfo['directLink'] = "https://www.youtube.com/embed/".$vid."?rel=0&wmode=transparent";
			    $vInfo['thumbnail'] = 'https://img.youtube.com/vi/' . $vid . '/0.jpg';

			    break;
		    case 'vimeo':
			    $hash = unserialize(@file_get_contents("https://vimeo.com/api/v2/video/$vid.php"));
			    if (!$hash) return false;

			    $vInfo['title'] = trim($hash[0]['title']);
			    $vInfo['id'] = $vid;
			    $vInfo['thumbnail'] = $hash[0]['thumbnail_large'];
			    $vInfo['type'] = 'vimeo';
			    $vInfo['directLink']  = "https://player.vimeo.com/video/". $vid;

		    	break;
		    default:
		    	break;
	    }
	    return $vInfo;
    }
}

Class CkStyle {
	public  $name;
	public  $element;
	public  $attributes;
	public  $styles;
}