<?php

/**
 * Dropeditor
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropeditor
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */
// No direct access.
defined('_JEXEC') or die;

jimport('joomla.application.component.modeladmin');

/**
 * Seoglossary model.
 */
class DropeditorModelBulleted extends JModelAdmin {

    /**
     * @var		string	The prefix to use with controller messages.
     * @since	1.6
     */
    protected $text_prefix = 'COM_DROPEDITOR';

    /**
     * Returns a reference to the a Table object, always creating it.
     *
     * @param	type	The table type to instantiate
     * @param	string	A prefix for the table class name. Optional.
     * @param	array	Configuration array for model. Optional.
     * @return	JTable	A database object
     * @since	1.6
     */
    public function getTable($type = 'Bulleted', $prefix = 'DropeditorTable', $config = array()) {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to get the record form.
     *
     * @param	array	$data		An optional array of data for the form to interogate.
     * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
     * @return	JForm	A JForm object on success, false on failure
     * @since	1.6
     */
    public function getForm($data = array(), $loadData = true) {
        // Initialise variables.
        $app = JFactory::getApplication();

        // Get the form.
        jimport('joomla.form.form');
        JForm::addFieldPath('JPATH_ADMINISTRATOR/components/com_dropeditor/models/fields');
        $form = $this->loadForm('com_dropeditor.format', 'format', array('control' => 'jform', 'load_data' => $loadData));
        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Method to get the data that should be injected in the form.
     *
     * @return	mixed	The data for the form.
     * @since	1.6
     */
    protected function loadFormData() {
        // Check the session for previously entered form data.
        $data = JFactory::getApplication()->getUserState('com_dropeditor.edit.format.data', array());

        if (empty($data)) {
            $data = $this->getItem();
        }

        return $data;
    }

    public function deleteBulleted($id) {
        $db = JFactory::getDbo();
        $q = "Delete From #__dropeditor_bulleteds WHERE id=" . (int) $id;
        $db->setQuery($q);

        return $db->execute();
    }

    public function getBulleted($id) {
        $db = JFactory::getDbo();
        $q = "Select * From #__dropeditor_bulleteds WHERE id=" . $id;
        $db->setQuery($q);
        return $db->loadObject();
    }

    public function addBulleted($data) {
        $dbo = $this->getDbo();
        $params = json_encode($data['params']);

        $dbo->setQuery('INSERT INTO #__dropeditor_bulleteds'
                . '(icon,title, params) '
                . 'VALUES '
                . '(' . $dbo->quote($data['icon']) . ',' . $dbo->quote($data['title']) . ',' .  $dbo->quote($params) . ')');
        if (!$dbo->execute()) {
            return false;
        }
        return $dbo->insertid();
    }
    
    public function saveBulleted($data) {
        $dbo = $this->getDbo();        
        $params = json_encode($data['params']);
        $dbo->setQuery('Update #__dropeditor_bulleteds '
                . 'SET icon='. $dbo->quote($data['icon']). ',title= ' .  $dbo->quote($data['title']) 
                .', params=' . $dbo->quote($params) 
                . ' WHERE id='. $data['id'] );
                    
        if(!$dbo->execute()){
            return false;
        }
        return true;
    }
    
    public function getItems() {
        $db = JFactory::getDbo();
        $q = "Select * From #__dropeditor_bulleteds Order by title";
        $db->setQuery($q);
        $rows = $db->loadObjectList();
        if(count($rows)) {
            foreach ($rows as $row) {
                $row->css = $this->generateCss($row->params);
                $row->attr = '';                                
                $row->cssClass =  $row->icon;             
            }
        }
        return $rows;
    }

    public function generateCss($param) {
        $css = array();        
        $data = json_decode($param);
        if(isset($data->color)) {
            $css[] = 'background-color:'.$data->color ;
            $bgcolor = $data->color;
        }else {
            $bgcolor = '#0e83cd';
        }
        if(isset($data->font_size)) {
            $css[] = 'font-size:'.$data->font_size .'px' ;
        }
        if(isset($data->padding_left)) {
            $css[] = 'padding-left:'.$data->padding_left .'px' ;
        }
        if(isset($data->padding_top)) {
            $css[] = 'padding-top:'.$data->padding_top .'px' ;
        }
        if(isset($data->padding_right)) {
            $css[] = 'padding-right:'.$data->padding_right .'px' ;
        }
        if(isset($data->padding_bottom)) {
            $css[] = 'padding-bottom:'.$data->padding_bottom .'px' ;
        }
        
        if(isset($data->btnStyle)) {
            switch ($data->btnStyle) {
                case 'btn1':
                    $borderColor  = $this->ColorLuminance($bgcolor, -0.2);                    
                    $css[] = 'border: 1px solid '. $borderColor;                            
                    break;
                case 'btn2':                                        
                    $css[] = 'border: 2px solid '. $bgcolor;                            
                    break;
                case 'btn5':                                                                                     
                    $hoverColor  = $this->ColorLuminance($bgcolor, -0.2);                 
                    $css[] = 'box-shadow: 0px 6px '. $hoverColor;   
                    break;
                case 'btn6':    
                    $hoverColor  = $this->ColorLuminance($bgcolor, -0.2);                 
                    $css[] = 'box-shadow: 0px 6px '. $hoverColor;                      
                    break;
               
                default:
                    break;
            }
        }
        
        return implode(';', $css);
    }
    
    function ColorLuminance($colourstr, $steps) {
        $colourstr = str_replace('#','',$colourstr);
        $rhex = substr($colourstr,0,2);
        $ghex = substr($colourstr,2,2);
        $bhex = substr($colourstr,4,2);

        $r = hexdec($rhex);
        $g = hexdec($ghex);
        $b = hexdec($bhex);

        $r = max(0,min(255,$r + $r*$steps));
        $g = max(0,min(255,$g + $g *$steps));  
        $b = max(0,min(255,$b + $b*$steps));

        return '#'.dechex($r).dechex($g).dechex($b);
    }
   
    /**
     * Method to get a single record.
     *
     * @param	integer	The id of the primary key.
     *
     * @return	mixed	Object on success, false on failure.
     * @since	1.6
     */
    public function getItem($pk = null) {
        if ($item = parent::getItem($pk)) {

            //Do any procesing on fields here if needed
        }

        return $item;
    }

    /**
     * Prepare and sanitise the table prior to saving.
     *
     * @since	1.6
     */
    protected function prepareTable($table) {
        jimport('joomla.filter.output');

        if (empty($table->id)) {
            // Set ordering to the last item if not set
            if (@$table->ordering === '') {
                $db = JFactory::getDbo();
                $db->setQuery('SELECT MAX(ordering) FROM #__dropeditor_styles');
                $max = $db->loadResult();
                $table->ordering = $max + 1;
            }
        }
    }
}
