<?php

/**
 * Dropeditor
 * 
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropeditor
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// No direct access.
defined('_JEXEC') or die;

jimport('joomla.application.component.modeladmin');

/**
 * Seoglossary model.
 */
class DropeditorModelFormat extends JModelAdmin
{
	/**
	 * @var		string	The prefix to use with controller messages.
	 * @since	1.6
	 */
	protected $text_prefix = 'COM_DROPEDITOR';


	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 * @return	JTable	A database object
	 * @since	1.6
	 */
	public function getTable($type = 'Style', $prefix = 'DropeditorTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		An optional array of data for the form to interogate.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	JForm	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true)
	{
		// Initialise variables.
		$app	= JFactory::getApplication();

		// Get the form.
		jimport('joomla.form.form');
		JForm::addFieldPath('JPATH_ADMINISTRATOR/components/com_dropeditor/models/fields');
		$form = $this->loadForm('com_dropeditor.format', 'format', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}

		return $form;
	}

       
        
	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */
	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_dropeditor.edit.format.data', array());

		if (empty($data)) {
			$data = $this->getItem();
		}

		return $data;
	}
        
	public function getCustomStyles() {
	    $db = JFactory::getDbo();
	    $q = "Select * From #__dropeditor_customstyles WHERE state=1 Order by ordering";
	    $db->setQuery($q);
	    return $db->loadObjectList();
	}
        
    public function deleteStyle($id) {
        $db = JFactory::getDbo();
        $q = "Delete From #__dropeditor_customstyles WHERE id=" . (int)$id;
        $db->setQuery($q);

        return $db->execute();
    }
        
	public function copyStyle($id){
        $table = $this->getTable( 'CustomStyle', 'DropeditorTable');
        $dbo = $this->getDbo();
        $dbo->setQuery('SELECT MAX(ordering) FROM #__dropeditor_customstyles');
        $max = $dbo->loadResult();

        if($table->load($id)){
            $table->id = null;
            $table->title = $table->title.JText::_('COM_DROPEDITOR_MODEL_COPY');
            $table->ordering =  $max+1;
            $table->store();

            $table->name = $table->name.$table->id;
            $table->store();

            return $table->id;
        }
        return false;
    }
        
    public function getCustomStyle($id) {
        $db = JFactory::getDbo();
        $q = "Select * From #__dropeditor_customstyles WHERE id=". $id;
        $db->setQuery($q);
        return $db->loadObject();
    }
        
    public function setStyleClass($id,$name){
        if($name==''){
            return false;
        }
        $filter = JFilterInput::getInstance();
        $name = $filter->clean($name);

        $table = $this->getTable( 'CustomStyle', 'DropeditorTable');
        if(!$table->load($id)){
            return false;
        }
        if(!$table->bind(array('name'=>$name))){
            return false;
        }
        if(!$table->store()){
            return false;
        }
        return true;
    }
        
    public function addCustomStyle($data = null) {
        $dbo = $this->getDbo();        
        $dbo->setQuery('SELECT MAX(ordering) FROM #__dropeditor_customstyles');
		$max = $dbo->loadResult();

		if (!$data) {
			$dbo->setQuery( 'INSERT INTO #__dropeditor_customstyles '
			                . '(title, name, state, ordering) '
			                . 'VALUES '
			                . '(' . $dbo->quote( JText::_( 'COM_DROPEDITOR_MODEL_FORMAT_DEFAULT_TITLE' ) ) . ',' . $dbo->quote( JText::_( 'COM_DROPEDITOR_MODEL_FORMAT_DEFAULT_CLASS' ) ) . ',1,' . ( $max + 1 ) . ')' );
			if ( ! $dbo->execute() ) {
				return false;
			}
		} else {
			$dbo->setQuery('INSERT INTO #__dropeditor_customstyles '
		                      . '(title, name,css, state, ordering) '
		                      . 'VALUES '
		                      . '('.$dbo->quote($data['title']).','.$dbo->quote($data['name']).','.$dbo->quote($data['css']).',1,'.($max + 1).')');
			if (!$dbo->execute()) {
				return false;
			}
		}
        return $dbo->insertid();
    }
        
       
    public function saveCustomStyle($data) {
        $dbo = $this->getDbo();        
      
        $dbo->setQuery('Update #__dropeditor_customstyles '
                . 'SET title= ' .  $dbo->quote($data['title']) .', css=' . $dbo->quote($data['css']) 
                . ' WHERE id='. $data['id'] );
                    
        if(!$dbo->execute()){
            return false;
        }
        return true;
    }
    
    public function getItems($onlyPublished=true) {
        $db = JFactory::getDbo();
        if($onlyPublished) {
            $q = "Select * From #__dropeditor_styles WHERE state=1 Order by ordering";
        }else {
            $q = "Select * From #__dropeditor_styles Order by ordering";
        }

        $db->setQuery($q);
        return $db->loadObjectList();
    }
        
	/**
	 * Method to get a single record.
	 *
	 * @param	integer	The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 * @since	1.6
	 */
	public function getItem($pk = null)
	{
		if ($item = parent::getItem($pk)) {

			//Do any procesing on fields here if needed

		}

		return $item;
	}

	/**
	 * Prepare and sanitise the table prior to saving.
	 *
	 * @since	1.6
	 */
	protected function prepareTable($table)
	{
		jimport('joomla.filter.output');

		if (empty($table->id)) {
			// Set ordering to the last item if not set
			if (@$table->ordering === '') {
				$db = JFactory::getDbo();
				$db->setQuery('SELECT MAX(ordering) FROM #__dropeditor_styles');
				$max = $db->loadResult();
				$table->ordering = $max+1;
			}
		}
	}

}