<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * Import/Export model class
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       2.2
 */
class DropeditorModelImportExport extends JModelLegacy
{
	/**
	 * Method to get data for exporting file
	 *
	 * @param   (string)    $table    Table to load data
	 * @param   (string)    $id_list  List of id to get, separate by comma
	 *
	 * @return mixed    Loaded data
	 */
	public function getData($table, $id_list = '') {
		$dbo = JFactory::getDbo();
		$query = $dbo->getQuery(true);
		$query->select('*');

		if ($table == 'profiles') {
			$query->from('#__dropeditor_profiles');
		} elseif ($table == 'custom_styles') {
			$query->from('#__dropeditor_customstyles');
		}

		$query->where('state != -2');

		if ($id_list && $id_list != 'all') {
			$query->where('id IN ('. $id_list.')');
		} else {
			if ($table == 'profiles' && $id_list != 'all') {
				return null;
			}
		}

		$dbo->setQuery($query);
		$data = $dbo->loadObjectList();

		// Add an identify property to our data
		if ($table == 'profiles') {
			$data['file_type'] = 'profiles';
		} elseif ($table == 'custom_styles') {
			$data['file_type'] = 'custom_styles';
		}

		return $data;
	}

	/**
	 * Method to import profiles from file
	 *
	 * @param $data     Data that loaded from file
	 *
	 * @return bool     True on success. False on failure
	 */
	public function importProfiles($data) {
		foreach ($data as $profile) {
			$dbo = JFactory::getDbo();
			$dbo->setQuery('SELECT MAX(ordering) FROM #__dropeditor_customstyles');
			$max = $dbo->loadResult();

			$query = $dbo->getQuery(true);
			$query->insert('#__dropeditor_profiles');
			$query->columns('`title`, `description`, `usergroups` ,`components`, `state`, `ordering`, `params`');
			$query->values($dbo->quote($profile['title']).','.$dbo->quote($profile['description']).','.$dbo->quote($profile['usergroups']).','.$dbo->quote($profile['components']).','.$dbo->quote($profile['state']).','.($max + 1).','.$dbo->quote($profile['params']));

			$dbo->setQuery($query);
			if (!$dbo->execute()) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Method to import custom styles from file
	 *
	 * @param $data         Data that loaded from import file
	 * @param $override     Option to override existing custom styles
	 *
	 * @return bool         True on success. False on failure
	 */
	public function importCustomStyles($data, $override) {
		$controller = JControllerLegacy::getInstance('Dropeditor');
		$formatModel = $controller->getModel('format');
		$saved_styles = $formatModel->getCustomStyles();

		// Explore old saved styles to find which class-name already exist
		foreach ($saved_styles as $old_style) {
			// Explore new import styles to find that class-name
			foreach ($data as $key => $style) {
				// If class-name exist
				if ($old_style->name == $style['name']) {
					$style['id'] = $old_style->id;
					// If user choose to override
					if ($override) {
						if ($formatModel->saveCustomStyle($style)) {
							unset($data[$key]);
						} else {
							return false;
						}
					} else { // If not, we ignore it
						unset($data[$key]);
					}
					break;
				}
			}
		}

		// Get the remain unsaved custom styles from imported file
		foreach ( $data as $unsaved_style ) {
			if (!$formatModel->addCustomStyle($unsaved_style)) {
				return false;
			};
		}

		DropeditorHelper::writeCustomStyle($formatModel->getCustomStyles());
		return true;
	}
}