<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

jimport('joomla.application.component.modeladmin');
JLoader::register('DropfilesFilesHelper', JPATH_ADMINISTRATOR . '/components/com_dropfiles/helpers/files.php');

/**
 * Link manager model class
 *
 * @package     Joomla.Administrator
 * @subpackage  com_dropeditor
 * @since       1.6
 */
class DropeditorModelLink extends JModelAdmin {

	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 * @return	JTable	A database object
	 * @since	1.6
	 */
	public function getTable($type = 'Link', $prefix = 'DropeditorLink', $config = array()) {
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		An optional array of data for the form to interogate.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	JForm	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true) {
		// Initialise variables.
		$app = JFactory::getApplication();

		// Get the form.
		jimport('joomla.form.form');
		JForm::addFieldPath('JPATH_ADMINISTRATOR/components/com_dropeditor/models/fields');
		$form = $this->loadForm('com_dropeditor.format', 'format', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}

		return $form;
	}

	/**
	 * Method to get items for tree view
	 *
	 * @param null string   $comp       Component that we get items from
	 * @param null int      $parent_id  Parent ID to get child items
	 *
	 * @return array|mixed              Array of matched items
	 */
	public function getItems($comp = null, $parent_id = null) {
		$dbo = JFactory::getDbo();
		$query = '';
		if ($comp == 'com_flexicontent') $comp = 'com_content';
		if (!$parent_id) {
			if ($comp == 'com_k2') {
				$query = 'SELECT id AS pid, parent AS parent_id, name AS text FROM #__k2_categories WHERE parent = 0 and published = 1 ORDER BY id';
			} else if ($comp == 'com_dropfiles') {
				$query = 'SELECT c.id AS pid, c.parent_id, c.title AS text FROM #__categories AS c INNER JOIN #__dropfiles as d ON c.id = d.id WHERE c.extension = "'.$comp.'" AND c.published = 1 AND c.parent_id = 1 AND (d.type = "default" OR d.type = "") ORDER BY c.lft';
			} else {
				$query = 'SELECT id AS pid, parent_id, title AS text FROM #__categories WHERE extension = "'.$comp.'" AND published = 1 AND parent_id = 1 ORDER BY id';
			}
			$dbo->setQuery($query);
			$items = $dbo->loadObjectList();
			foreach ($items as $item) {
				$item->children = true;
				$item->type = 'category';
			}
		} else {
			// Get all child categories
			if ($comp == 'com_k2') {
				$query = 'SELECT id AS pid, parent AS parent_id, name AS text FROM #__k2_categories WHERE parent = '.$parent_id.' and published = 1 ORDER BY id';
			} else {
				$query = 'SELECT id AS pid, parent_id, title AS text FROM #__categories WHERE extension = "'.$comp.'" AND published = 1 AND parent_id = '.$parent_id.' ORDER BY lft';
			}
			$dbo->setQuery($query);
			$items = $dbo->loadObjectList();
			foreach ($items as $item) {
				$item->children = true;
				$item->type = 'category';
			}

			// Get all child items
			switch ($comp) {
				case 'com_content':
					$query = 'SELECT id AS aid , title AS text, catid AS parent_id FROM #__content WHERE catid = '.$parent_id.' AND state = 1 ORDER BY id';
					break;
				case 'com_dropfiles':
					$query = 'SELECT i.id AS aid, i.title, i.ext, i.catid , c.title AS cat_title FROM #__dropfiles_files AS i INNER JOIN #__categories AS c ON i.catid = c.id WHERE i.catid ='.$parent_id.' AND i.state = 1 ORDER BY i.ordering';
					break;
				case 'com_k2':
					$query = 'SELECT i.id as aid, i.title as text FROM #__k2_items as i LEFT JOIN #__k2_categories AS c ON c.id = i.catid LEFT JOIN #__viewlevels AS g ON g.id = i.access LEFT JOIN #__users AS u ON u.id = i.checked_out LEFT JOIN #__users AS v ON v.id = i.created_by LEFT JOIN #__users AS w ON w.id = i.modified_by WHERE i.trash=0 AND i.published=1 AND catid='.$parent_id.' ORDER BY i.id DESC';
					break;
			}
			$dbo->setQuery($query);
			$files = $dbo->loadObjectList();
			foreach ($files as $file) {
				$file->type = 'item';
				if ($comp == 'com_dropfiles') {
					$file->text = $file->title.'.'.$file->ext;
					$file_url = DropfilesFilesHelper::genUrl($file->aid, $file->catid, $file->cat_title, '',$file->text);
					$file->url = $file_url;
				}
			}

			// Append child items after child categories
			$items = array_merge($items, $files);
		}

		return $items;
	}
}